@@ -246,7 +246,7 @@
 #define DEFAULT_UINT1_PADVALUE          ((unsigned char)254)
 #define DEFAULT_INT2_PADVALUE           ((short)-32767)
 #define DEFAULT_UINT2_PADVALUE          ((unsigned short)65534)
-#define DEFAULT_INT4_PADVALUE           (-2147483647L)
+#define DEFAULT_INT4_PADVALUE           (-2147483647)
 #define DEFAULT_INT8_PADVALUE           ((long long)-9223372036854775807LL)
 #define DEFAULT_UINT4_PADVALUE          (4294967294U)
 #define DEFAULT_REAL4_PADVALUE          ((float)-1.0E30f)
@@ -270,7 +270,8 @@
 * Macintosh.
 *****************************************************************************/
 
-#if defined(vax) || defined(alphavmsD) || defined(alphavmsG)
+#if defined(vax) || defined(alphavmsD) || defined(alphavmsG) || \
+    defined(ia64vmsD) || defined(ia64vmsG)
 #  define DEFAULT_NEGtoPOSfp0             TRUE
 #else
 #  define DEFAULT_NEGtoPOSfp0             FALSE
@@ -318,10 +319,13 @@
 						   FALSE = "multi". */
 #define DEFAULTnetworkEXPORT            FALSE   /* TRUE = "network",
 						   FALSE = "host". */
-#define DEFAULTepochEXPORT              0       /* 0 (standard), 1, 2, 3, 4
-						   (C/Fortran format), 5
-						   (custom format), or 6
+#define DEFAULTepochEXPORT              4       /* 0 (standard), 1, 2, 3, 4. */
+#define DEFAULTstyleEXPORT              6       /* For CDF_EPOCH/CDF_EPOCH16.
+                                                   Style value: 0, 1, 2, 3, 4 
+                                                   (C/Fortran format), 5
+                                                   (custom format), or 6
                                                    (ISO 8601). */
+#define DEFAULTtt2000EXPORT             0       /* 0, 1, 2, 3 (ISO 8601). */
 #define DEFAULThorizontalEXPORT         FALSE   /* TRUE = "horizontal",
 						   FALSE = "vertical". */
 #define DEFAULTmajorityEXPORT           0       /* ROW_MAJOR, COLUMN_MAJOR, or
@@ -335,12 +339,13 @@
 						   enabled? */
 
 #define DEFAULTpromptSIMPLE		TRUE	/* Don't change these... */
-#define DEFAULTformatSIMPLE		TRUE
+#define DEFAULTformatSIMPLE		FALSE
 #define DEFAULThorizontalSIMPLE		TRUE
 #define DEFAULTrecordSIMPLE		FALSE
 #define DEFAULTindicesSIMPLE		FALSE
 #define DEFAULToutputSIMPLE		FALSE
-#define DEFAULTepochSIMPLE		0
+#define DEFAULTepochSIMPLE		6
+#define DEFAULTtt2000SIMPLE		3
 #define DEFAULTmajoritySIMPLE		0
 #define DEFAULTspacingSIMPLE		1
 #define DEFAULTheadingSIMPLE		TRUE
@@ -364,6 +369,12 @@
 						   zMODEon2 = zMode/2 */
 #define DEFAULTgWithEDIT        FALSE           /* [no]gwithentries */
 #define DEFAULTvWithEDIT        FALSE           /* [no]vwithentries */
+#define DEFAULTstyleEDIT        6               /* EPOCH4_STYLE */
+                                                /* For CDF_EPOCH and
+                                                   CDF_EPOCH16. Value: 0, 1,
+                                                   2, 3, 4 (C/Fortran format),
+                                                   5, (custom format), or 6
+                                                   (ISO 8601). */
 
 /******************************************************************************
 * Command line defaults for CDFconvert.
@@ -391,6 +402,7 @@
 
 #define DEFAULTpageINQ          FALSE           /* [no]page */
 #define DEFAULTidINQ            TRUE            /* TRUE if /ID,-id specified.*/
+#define DEFAULTstyleINQ         6               /* EPOCH4_STYLE */
 
 /******************************************************************************
 * Command line defaults for CDFirsdump.
@@ -409,6 +421,7 @@
 						   2 == "most",
 						   3 == "full" */
 #define DEFAULTdataIRsDUMP	   FALSE	/* data. */
+#define DEFAULTstyleIRsDUMP	   6	        /* EPOCH4_STYLE. */
 
 /******************************************************************************
 * Command line defaults for CDFcompare.
@@ -425,6 +438,7 @@
 #define DEFAULTpctCMP           FALSE           /* [no]percent */
 #define DEFAULTetcCMP           TRUE            /* [no]etc */
 #define DEFAULTlocationCMP      FALSE           /* [no]location */
+#define DEFAULTdataCMP          TRUE            /* [no]data */
 #define DEFAULTvalueCMP         FALSE           /* [no]value */
 #define DEFAULTformatCMP        TRUE            /* [no]format */
 #define DEFAULTstatsCMP         FALSE           /* [no]statistics */
@@ -433,6 +447,7 @@
 						   zMODEon2 = zMode/2.
 						   This default applies to
 						   both CDFs being compared. */
+#define DEFAULTstyleCMP         6               /* EPOCH4_STYLE */
 
 /******************************************************************************
 * Command line defaults for CDFstats.
@@ -452,6 +467,7 @@
 #define DEFAULTzModeSTATS       zMODEoff        /* zMODEoff = zMode/0,
 						   zMODEon1 = zMode/1,
 						   zMODEon2 = zMode/2 */
+#define DEFAULTstyleSTATS       6               /* EPOCH4_STYLE */
 
 /******************************************************************************
 * Command line defaults for SkeletonTable.
@@ -475,6 +491,7 @@
 						   to skeleton table file. */
 #define DEFAULTpageCDF2SKT      FALSE           /* [no]page */
 #define DEFAULTstatsCDF2SKT	FALSE		/* [no]statistics */
+#define DEFAULTstyleCDF2SKT	0		/* EPOCH0_STYLE */
 
 /******************************************************************************
 * Command line defaults for SkeletonCDF.
@@ -503,6 +520,7 @@
 #define DEFAULTheaderDUMP       TRUE            /* [no]header */
 #define DEFAULTcol2rowDUMP      FALSE           /* col2row    */
 #define DEFAULTepochVVDUMP      FALSE           /* epochVV    */
+#define DEFAULTstyleDUMP        6               /* EPOCH4_STYLE */
 
 /******************************************************************************
 * Command line defaults for CDFmerge.
