@@ -59,6 +59,8 @@
 *                               zlib.
 *   V2.18 09-Aug-13, M Liu      Added cdfid_AllocateMemory, cdfid_getCDFid,
 *                               cdfid_FreeMemory.
+*   V2.19 09-Jan-18, M Liu      Added ARM and IA64 openVMS architectures
+*                               support.
 *
 ******************************************************************************/
 
@@ -78,8 +80,10 @@
 #      define MSVC67 
 #    endif
 #  endif
-#  if defined (_MSC_VER)
-#    define snprintf _snprintf
+#  if defined (_MSC_VER) && (_MSC_VER < 1900)
+#     define snprintf _snprintf
+#     define isnan(x) _isnan(x)
+#     define isinf(x) (!_finite(x))
 #  endif
 #endif
 
@@ -112,7 +116,8 @@
 #  define Mach
 #endif
 
-#if defined(__ppc__) || (defined(__APPLE__) && !defined(i386))
+#if defined(__ppc__) || (defined(__APPLE__) && !defined(i386) && \
+                         !defined(__x86_64__))
 #  define POWERPC
 #  define unix
 #  if defined(__MACH__)  	/* Mac OS X cc (gcc) */
@@ -120,7 +125,7 @@
 #  endif
 #endif
 
-#if (defined(__APPLE__) && defined(i386))
+#if defined(__APPLE__) && (defined(i386) || defined(__x86_64__))
 #  define unix
 #  define IBMPC
 #  define macosXintel
@@ -130,6 +135,30 @@
 #  define unix
 #endif
 
+#if defined(__arm__)
+#  define ARM
+#  if defined(__BYTE_ORDER__)
+#     if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+#        define ARM_LITTLE
+#     else
+#        define ARM_BIG
+#     endif
+#  endif
+#endif
+
+#if defined(vms) && defined(__ia64__)
+#  define vmsIA64
+#  if __D_FLOAT
+#    define ia64vmsD
+#  endif
+#  if __G_FLOAT
+#    define ia64vmsG
+#  endif
+#  if __IEEE_FLOAT
+#    define ia64vmsI
+#  endif
+#endif
+
 #if defined(linux) || defined(__CYGWIN__) || defined(__MINGW32__) || \
     defined(__FreeBSD__)
 #  if defined(PPC) || defined(__PPC__) 
@@ -273,6 +302,7 @@
 #  include <smgdef.h>
 #  include <ssdef.h>
 #  include <stat.h>
+#  include <unistd.h>
 #  include <lib$routines.h>
 #  if defined(VARARGS)
 #    include <varargs.h>
@@ -308,6 +338,10 @@
 #  endif
 #endif
 
+#if defined(__MACH__)
+#  include <mach-o/dyld.h>
+#endif
+
 #if defined(dos) || defined(win32) || defined(__MINGW32__)
 #  include <stdlib.h>
 #  include <stdio.h>
@@ -318,6 +352,7 @@
 #  if defined(win32)
 #    include <io.h>
 #    include <fcntl.h>
+/*#    include <Windows.h> */
 #  endif
 #  if defined(BORLANDC)
 #    include <dir.h>
@@ -327,6 +362,7 @@
 #  endif
 #  if defined(__MINGW32__)
 #    include <dirent.h>
+#    include <unistd.h>
 #  endif
 #  include <conio.h>
 #  include <sys\types.h>
@@ -411,7 +447,7 @@ typedef unsigned short uShort;
 typedef unsigned int uInt4;
 typedef unsigned long uLongx;
 
-#if defined(alphaosf) || defined(IRIX64bit) || defined(SOLARIS64) || \
+#if defined(alphaosf) || defined(IRIX64bit) || defined(_LP64) || \
     defined(__amd64) || defined(__x86_64__) || defined(__ia64__) || \
     defined(AIX64) || defined(HP64) || defined(__PPC64__) || \
     defined(__ppc64__)
@@ -421,17 +457,24 @@ typedef unsigned long uLongx;
    * They are also 64-bits on AIX, HP-UX and PPC if 64-bit mode is on.
    * `int' is 32 bits in these cases.
    ***************************************************************************/
+/*
    typedef int Int32;
    typedef uInt4 uInt32;
    typedef long Int64;
+*/
 #else
+/*
    typedef long Int32;
-   typedef uLongx uInt32;
+   typedef uInt4 uInt32;
    typedef long long Int64;
+*/
 #endif
 
 typedef short Int16;
 typedef uShort uInt16;
+typedef int Int32;
+typedef uInt4 uInt32;
+typedef long long Int64;
 
 #if defined(AIX) || defined(dos) || defined(sgi) || defined(__QNX__)
    /***************************************************************************
@@ -521,7 +564,11 @@ typedef sChar sByte;
 #    if defined(__CYGWIN__) || defined(vms)
 #      define STAT     stat
 #    else
-#      define STAT     stat64
+#      if defined(__DARWIN_64_BIT_INO_T)
+#         define STAT     stat
+#      else
+#         define STAT     stat64
+#      endif
 #    endif
 #  endif
 #else
@@ -567,8 +614,9 @@ typedef sChar sByte;
 * Floating-point encodings.
 *   1..........Sun, SGi, IBM-RS, HP, NeXT, Macintosh, PowerPC
 *   2..........DECstation, IBM-PC, Alpha (OSF/1), Alpha (OpenVMS - IEEE_FLOAT)
-*   3..........VAX, Alpha (OpenVMS - D_FLOAT)
-*   4..........Alpha (OpenVMS - G_FLOAT)
+*              ARM, IA64 (OpenVMS - IEEE_FLOAT)
+*   3..........VAX, Alpha and IA64 (OpenVMS - D_FLOAT)
+*   4..........Alpha and IA64 (OpenVMS - G_FLOAT)
 *****************************************************************************/
 
 #if defined(sun) || defined(MIPSEB) || defined(IBMRS) || defined(HP) || \
@@ -577,15 +625,18 @@ typedef sChar sByte;
 #endif
 
 #if defined(MIPSEL) || defined(IBMPC) || defined(alphaosf) || \
-    defined(alphavmsI) || defined(posixSHELLalphaI)
+    defined(alphavmsI) || defined(posixSHELLalphaI) || defined(ARM) || \
+    defined(ia64vmsI)
 #  define FP2cpu
 #endif
 
-#if defined(vax) || defined(alphavmsD) || defined(posixSHELLalphaD)
+#if defined(vax) || defined(alphavmsD) || defined(posixSHELLalphaD) || \
+    defined(ia64vmsD)
 #  define FP3cpu
 #endif
 
-#if defined(alphavmsG) || defined(posixSHELLalphaG)
+#if defined(alphavmsG) || defined(posixSHELLalphaG) || \
+    defined(ia64vmsG)
 #  define FP4cpu
 #endif
 
@@ -656,7 +707,8 @@ ap = _apT_; \
 #endif
 
 #if defined(unix)
-#  if (defined(sun) && !defined(SOLARIS)) || defined(sgi)
+#  if (defined(sun) && (!defined(_SUNOS_VTOC_8) && !defined(_SUNOS_VTOC_16))) \
+      || defined(sgi)
 #    define memmove(dst,src,nbytes) bcopy(src,dst,nbytes)
 #  endif
 #endif
@@ -786,6 +838,42 @@ ap = _apT_; \
 #define STRINGdataType(dataType) \
 (dataType == CDF_CHAR || dataType == CDF_UCHAR)
 
+/******************************************************************************
+* CDFstringDataType.
+*    Macro to check if a character string data type.
+******************************************************************************/
+
+#define CDFstringDataType(dataType) \
+(dataType == CDF_CHAR || dataType == CDF_UCHAR)
+
+/******************************************************************************
+* CDFintDataType.
+*    Macro to check if one of the CDF integer data types.
+******************************************************************************/
+
+#define CDFintDataType(dataType) \
+(dataType == CDF_INT1 || dataType == CDF_BYTE || dataType == CDF_UINT1 || \
+ dataType == CDF_INT2 || dataType == CDF_UINT2 || dataType == CDF_INT4 || \
+ dataType == CDF_UINT4 || dataType == CDF_INT8)
+
+/******************************************************************************
+* CDFfloatDataType.
+*    Macro to check if one of the CDF float data types.
+******************************************************************************/
+
+#define CDFfloatDataType(dataType) \
+(dataType == CDF_REAL4 || dataType == CDF_REAL8 || dataType == CDF_FLOAT || \
+ dataType == CDF_DOUBLE)
+
+/******************************************************************************
+* CDFepochDataType.
+*    Macro to check if one of the CDF epoch data types.
+******************************************************************************/
+
+#define CDFepochDataType(dataType) \
+(dataType == CDF_EPOCH || dataType == CDF_EPOCH16 || \
+ dataType == CDF_TIME_TT2000)
+
 /******************************************************************************
 * EPOCHdataType.
 *    Macro to check if an EPOCH data type.
@@ -903,6 +991,8 @@ typedef struct vSTATSstruct {
 #define CURENTRY_EXISTENCE(eT) E3(eT,CURgENTRY_EXISTENCE_, \
 				     CURrENTRY_EXISTENCE_, \
 				     CURzENTRY_EXISTENCE_)
+#define ENTRY_NUMSTRINGS(zOp) BOO(zOp,zENTRY_NUMSTRINGS_,rENTRY_NUMSTRINGS_)
+#define ENTRY_STRINGDATA(zOp) BOO(zOp,zENTRY_STRINGDATA_,rENTRY_STRINGDATA_)
 
 /******************************************************************************
 * Variable items...
@@ -991,7 +1081,6 @@ VISIBLE_PREFIX char *strcatX PROTOARGs((char *dst, char *src, size_t max));
   }
 #endif
 
-VISIBLE_PREFIX int CDFelemSize PROTOARGs((long));
 VISIBLE_PREFIX void ClearBytes PROTOARGs((
   void *buffer, int firstByte, int lastByte
 ));
