@@ -141,11 +141,11 @@
 
 #define TIME_STAMP_LEN		31
 
-#define EPOCH0_STYLE		0		/* Standard */
+#define EPOCH0_STYLE		0
 #define EPOCH1_STYLE		1
 #define EPOCH2_STYLE		2
 #define EPOCH3_STYLE		3
-#define EPOCH4_STYLE		6
+#define EPOCH4_STYLE		6		/* ISO 8601 -- new standard */
 #define EPOCHf_STYLE		4		/* Use C/Fortran format. */
 #define EPOCHx_STYLE		5		/* Use custom format. */
 
@@ -153,10 +153,11 @@
 * TT2000/time constants.
 *****************************************************************************/
 
-#define TT2000_0_STYLE		0               /* Standard */
+#define TT2000_0_STYLE		0
 #define TT2000_1_STYLE		1
 #define TT2000_2_STYLE		2
-#define TT2000_3_STYLE		3
+#define TT2000_3_STYLE		3		/* ISO 8601 -- Standard */
+#define TT2000_4_STYLE		4		/* ISO 8601 -- Standard */
 
 /*****************************************************************************
 * QOP constants.
@@ -189,7 +190,7 @@ char *qualOpt[QOP_MAX_QUALs];
 * Format specifiers.
 ******************************************************************************/
 
-#if defined(alphaosf) || defined(IRIX64bit) || defined(SOLARIS64) || \
+#if defined(alphaosf) || defined(IRIX64bit) || defined(_LP64) || \
     defined(__amd64) || defined(__x86_64__) || defined(__ia64__) || \
     defined(__PPC64__) || defined(__ppc64__)
 #  define Int32FORMAT	"%d"
@@ -197,27 +198,33 @@ char *qualOpt[QOP_MAX_QUALs];
 #  define Int32uFORMAT	"%u"
 #  define Int32FORMATmod	""
 #  define Int64FORMATmod        "ll"
-#  define Int32hexFORMAT	"0x%08X"
+#  define Int32hexFORMAT0x	"0x%08X"
+#  define Int32hexFORMAT	"%08X"
 #  if defined(alphaosf)
-#    define Int64hexFORMAT      "0x%016X"
+#    define Int64hexFORMAT0x    "0x%016X"
+#    define Int64hexFORMAT      "%016X"
 #  else
-#    define Int64hexFORMAT      "0x%016llX"
+#    define Int64hexFORMAT0x    "0x%016llX"
+#    define Int64hexFORMAT      "%016llX"
 #  endif
 #else
-#  define Int32FORMAT	"%ld"
+#  define Int32FORMAT	"%d"
 #if defined(win32) || defined(__MINGW32__)
 #  define Int64FORMAT   "%I64d"
 #else
 #  define Int64FORMAT   "%lld"
 #endif
-#  define Int32uFORMAT	"%lu"
+#  define Int32uFORMAT	"%u"
 #  define Int32FORMATmod	"l"
 #  define Int64FORMATmod	"ll"
-#  define Int32hexFORMAT	"0x%08lX"
+#  define Int32hexFORMAT0x	"0x%08X"
+#  define Int32hexFORMAT	"%08X"
 #if defined(win32) || defined(__MINGW32__)
-#  define Int64hexFORMAT        "0x%016I64X"
+#  define Int64hexFORMAT0x      "0x%016I64X"
+#  define Int64hexFORMAT        "%016I64X"
 #else
-#  define Int64hexFORMAT        "0x%016llX"
+#  define Int64hexFORMAT0x      "0x%016llX"
+#  define Int64hexFORMAT        "%016llX"
 #endif
 #endif
 
@@ -337,7 +344,7 @@ struct HyperStruct {
 #define MAX_ENCODED_INT4_LEN		11
 #define MAX_ENCODED_INT8_LEN		20
 #define MAX_REPORTMASK_LEN		40
-#define MAX_oTEXT_LEN			150
+#define MAX_oTEXT_LEN			1024
 #define MAX_CACHESIZES_LEN		30
 #define MAX_TOLERANCES_LEN              30
 
@@ -646,7 +653,7 @@ extern int lastSecond;
 * Function prototypes.
 *****************************************************************************/
 
-int UTF8StrLength PROTOARGs((unsigned char *));
+int UTF8StrLength PROTOARGs((unsigned char *, int));
 void ASSIGNx PROTOARGs((void *, void *, long, long));
 Logical EQx PROTOARGs((void *, void *, long, long));
 Logical NEx PROTOARGs((void *, void *, long, long));
@@ -657,6 +664,7 @@ Logical GTx PROTOARGs((void *, void *, long, long));
 Logical IsDir PROTOARGs((char *));
 Logical IsWild PROTOARGs((char *));
 Logical IsCDF PROTOARGs((char *));
+Logical IsHexFormat PROTOARGs((char *));
 void EncodeRecordJustify PROTOARGs((char *string, long recN, int minWidth,
   size_t width
 ));
@@ -696,6 +704,7 @@ int EncodeRecordIndices PROTOARGs((char *, long, long, long, long *, long *,
 int EncodeRecordIndicesVARY PROTOARGs((char *, long, long, long *, size_t));
 int EncodeString PROTOARGs((long, char *, char *, int, int));
 int EncodeValue PROTOARGs((long, void *, char *, int, size_t));
+int EncodeValueSKT PROTOARGs((long, void *, char *, int, size_t));
 int EncodeValueFormat PROTOARGs((
   long dataType, void *binary, char *text, char *format, int minWidth,
   int maxWidth, int style, size_t width
@@ -706,7 +715,9 @@ int EncodeValuesFormat PROTOARGs((
 ));
 void EncodeNegativeZero PROTOARGs((char *string, char *format, size_t width));
 void WriteStringValue PROTOARGs((FILE *, long, void *, int, int));
-void WriteEntryValue PROTOARGs((FILE *, long, long, void *, int, int));
+void WriteEntryValue PROTOARGs((FILE *, long, long, void *, int, int, int, char *));
+void WriteEntryValueSKT PROTOARGs((FILE *, long, long, void *, int, int, int,
+                                   char *));
 char *EncodingToken PROTOARGs((long));
 char *MajorityToken PROTOARGs((long));
 char *FormatToken PROTOARGs((long));
@@ -719,6 +730,7 @@ char *SparsenessToken PROTOARGs((
   long sRecordsType, long sArraysType, long sArraysParms[CDF_MAX_PARMS]
 ));
 char *ChecksumToken PROTOARGs((long));
+int  IsFloatingValidValue PROTOARGs((long, void *));
 long WhichDataType PROTOARGs((char *));
 long WhichFormat PROTOARGs((char *));
 long WhichMajority PROTOARGs((char *));
@@ -776,7 +788,8 @@ Logical S6qualifierLong PROTOARGs((
   long defaultS, char *conflictText
 ));
 void BlankPadRight PROTOARGs((char *string, int count));
-CDFstatus GetFormatEntry PROTOARGs((Logical, long, char **));
+void NulPadRight PROTOARGs((char *string, int count));
+CDFstatus GetFORMATEntry PROTOARGs((CDFid, Logical, long, char **));
 Logical ParseOptionList PROTOARGs((
   int nTokens, char *tokens[], char *list, Logical present[]
 ));
@@ -792,6 +805,11 @@ int DirList PROTOARGs((
 int CDFdirList PROTOARGs((char *path, char ***dirS, char ***nameS));
 void RemoveExtensions PROTOARGs((int, char **));
 char *TempCharStr PROTOARGs((int character));
+char *EpochDateTimeString PROTOARGs((long datatype, int style, double *epoch));
+void GetDefaultPadValue PROTOARGs((CDFid id, long datatype, void *value));
+void ReplacePadValues PROTOARGs((void *buffer, void *pad, void *filled,
+                                 long dataType, int values
+));
 void DisplayStatistics PROTOARGs((
   char *label, vSTATS *vStatsDotCDF, vSTATS *vStatsStage,
   vSTATS *vStatsCompress
@@ -864,6 +882,9 @@ int GetNumTokens PROTOARGs((int token, char *string));
 Logical ValidateQual PROTOARGs((
   char *qual, char *validQuals[]
 ));
+void DisplayCDFdata PROTOARGs((
+  long dataType, long numElems, void *value
+));
 int BinarySearch PROTOARGs((
   long dataType, void *values, void *value, int min, int max
 ));
@@ -947,11 +968,6 @@ void DebugMessage PROTOARGs((char *text));
 }
 #endif
 
-#if defined(win32)
-#define isnan(x) _isnan(x)
-#define isinf(x) (!_finite(x))
-#endif
-
 #if defined(vms)
 #define isinf(x) (!finite(x))
 #endif
