@@ -248,6 +248,89 @@ void    *value;         /* Out -- Value. */
   return pStatus;
 }
 
+/******************************************************************************
+* CDFgetAttrEntryNumStrings.
+* Can't implement with macro because the attribute's scope determines which
+* item(s) to use.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFgetAttrEntryNumStrings (id, rzEntry, attrNum,
+                                                    entryNum, numStrings)
+CDFid   id;             /* In -- CDF id. */
+int     rzEntry;        /* In -- Flag for r/zEntry. */
+long    attrNum;        /* In -- Attribute number. */
+long    entryNum;       /* In -- gEntry/rEntry/zEntry number. */
+long    *numStrings;    /* Out -- number of strings. */
+{
+  long scope;
+  CDFstatus pStatus = CDF_OK;
+  if (!sX(CDFlib(SELECT_, CDF_, id,
+                          ATTR_, attrNum,
+                 NULL_), &pStatus)) return pStatus;
+  if (!sX(CDFlib(SELECT_, BOO((rzEntry == 1),zENTRY_,
+                              rENTRY_), entryNum,
+                 GET_, BOO((rzEntry == 1),zENTRY_NUMSTRINGS_,
+                           rENTRY_NUMSTRINGS_), numStrings,
+                 NULL_), &pStatus)) return pStatus;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFgetAttrEntryStrings.
+* Can't implement with macro because the attribute's scope determines which
+* item(s) to use.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFgetAttrEntryStrings (id, rzEntry, attrNum,
+                                                 entryNum, numStrings, strings)
+CDFid   id;             /* In -- CDF id. */
+int     rzEntry;        /* In -- Flag for r/zEntry. */
+long    attrNum;        /* In -- Attribute number. */
+long    entryNum;       /* In -- gEntry/rEntry/zEntry number. */
+long    *numStrings;    /* Out -- number of strings. */
+char    ***strings;     /* Out -- strings. */
+{
+  long scope;
+  CDFstatus pStatus = CDF_OK;
+  if (!sX(CDFlib(SELECT_, CDF_, id,
+                          ATTR_, attrNum,
+                 NULL_), &pStatus)) return pStatus;
+  if (!sX(CDFlib(SELECT_, BOO((rzEntry == 1),zENTRY_,
+                              rENTRY_), entryNum,
+                 GET_, BOO((rzEntry == 1),zENTRY_STRINGSDATA_,
+                           rENTRY_STRINGSDATA_), numStrings, strings,
+                 NULL_), &pStatus)) return pStatus;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFputAttrEntryStrings.
+* Can't implement with macro because the attribute's scope determines which
+* item(s) to use.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFputAttrEntryStrings (id, rzEntry, attrNum,
+                                                 entryNum, numStrings, strings)
+CDFid   id;             /* In -- CDF id. */
+int     rzEntry;        /* In -- Flag for r/zEntry. */
+long    attrNum;        /* In -- Attribute number. */
+long    entryNum;       /* In -- gEntry/rEntry/zEntry number. */
+long    numStrings;     /* In -- number of strings. */
+char    **strings;      /* In -- strings. */
+{
+  long scope;
+  CDFstatus pStatus = CDF_OK;
+  if (!sX(CDFlib(SELECT_, CDF_, id,
+                          ATTR_, attrNum,
+                 NULL_), &pStatus)) return pStatus;
+  if (!sX(CDFlib(SELECT_, BOO((rzEntry == 1),zENTRY_,
+                              rENTRY_), entryNum,
+                 PUT_, BOO((rzEntry == 1),zENTRY_STRINGSDATA_,
+                           rENTRY_STRINGSDATA_), numStrings, strings,
+                 NULL_), &pStatus)) return pStatus;
+  return pStatus;
+}
+
 /******************************************************************************
 * CDFdataFree.
 * Free the dynamic space if it exists.It applies to pointer of CDFdata type.
@@ -1739,13 +1822,6 @@ VISIBLE_PREFIX long CDFgetChecksumMode ()
 VISIBLE_PREFIX void CDFsetFileBackwardFlag (flag)
 int flag;
 {
-/*
-  long version;
-  CDFstatus status;
-  status = CDFlib (GET_, LIB_VERSION_, &version,
-                   NULL_);
-  if (version == 3) backward = flag;
-*/
   backward = flag;
 }
 
