@@ -129,13 +129,6 @@ switch (Va->item) {
     if (!CDF->singleFile) return UNSUPPORTED_OPERATION;
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
-/*
-    if (!sX(LocateCurrentVar(CDF,zOp,&vOffset,&zVar,NULL),&pStatus)) {
-      AbortAccess (CDF, UPDATE, noDELETE);
-      return pStatus;
-    }
-*/
-
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) vOffset = CDF->CURzVarOffset;
     else vOffset = CDF->CURrVarOffset;
