@@ -30,6 +30,10 @@
 *                               value is greater than 16, it causes
 *                               segmentation fault for the IDL CDF interface
 *                               routine.
+*   Note: Those offset fields for CURgrEntryOffset64, CURzEntryOffset64, 
+*         CURattrOffset64, etc, in CDF struture may not have the proper 
+*         values set if the file is open with READONLY mode. Their actual
+*         offsets are kept in a sepate buffers.
 ******************************************************************************/
 
 #include "cdflib.h"
@@ -56,7 +60,7 @@ switch (Va->item) {
     Int32 magicNumber1, magicNumber2; int i;
     long *cType, *cParms; 
     void *cFileSize, *uFileSize;
-    long cSize2, uSize2;
+    long cSize2 = 0L, uSize2 = 0L;
     OFF_T cSize3, uSize3;
     int whichVersion = 3;
     CDFpathP = va_arg (Va->ap, char *);
@@ -595,6 +599,23 @@ switch (Va->item) {
     break;
   }
   /****************************************************************************
+  * CDF_LEAPSECONDLASTUPDATED_, 
+  ****************************************************************************/
+  case CDF_LEAPSECONDLASTUPDATED_: {
+    struct CDFstruct *CDF;
+    long *lastUpdated = va_arg (Va->ap, long *);
+    Int32 tLastUpdated;
+    SelectCDF (Cur->cdf, CDF)
+    if (!sX(ReadGDR64(CDF->fp,CDF->GDRoffset64,
+		      GDR_LEAPSECONDLASTUPDATED,&tLastUpdated,
+		      GDR_NULL),&pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    *lastUpdated = tLastUpdated;
+    break;
+  }
+  /****************************************************************************
   * LIB_VERSION_, 
   ****************************************************************************/
   case LIB_VERSION_: {
@@ -1414,7 +1435,6 @@ switch (Va->item) {
     if (!sX(ReadVDR64(CDF,CDF->fp,offset,zVar,
 		      VDR_FLAGS,&flags,
 		      VDR_DATATYPE,&dataType,
-		      VDR_NUMELEMS,&numElems,
 		      VDR_NULL),&pStatus)) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -1426,9 +1446,7 @@ switch (Va->item) {
       of the pad value is greater than 16, it causes segmentation fault
       for the IDL CDF interface routine.
     ***********************************************************************/
-    if (dataType == CDF_CHAR ||  dataType == CDF_UCHAR) 
-          numElems = 1;
-
+    numElems = 1;
     if (PADvalueBITset(flags)) {
       if (!sX(ReadVDR64(CDF,CDF->fp,offset,zVar,
 	    	        VDR_PADVALUE,padValue,
@@ -1452,12 +1470,6 @@ switch (Va->item) {
         DefaultPadValuePre350 (dataType, numElems, padValue);
       else
         DefaultPadValue (dataType, numElems, padValue);
-      if (!sX(ConvertBuffer(HostEncoding(),CDF->decoding,
-			    CDF->negToPosFp0,dataType,
-			    numElems,padValue,padValue),&pStatus)) {
-	AbortAccess64 (CDF, UPDATE, noDELETE);
-	return pStatus;
-      }
       sX (NO_PADVALUE_SPECIFIED, &pStatus);
     }
     break;
@@ -1853,7 +1865,34 @@ switch (Va->item) {
     break;
   }
   /****************************************************************************
-  * gENTRY_DATA_/rENTRY_DATA_/zENTRY_DATA_, 
+  * rENTRY_NUMSTRINGS_/zENTRY_NUMSTRINGS_
+  ****************************************************************************/
+  case rENTRY_NUMSTRINGS_:
+  case zENTRY_NUMSTRINGS_: {
+    Logical dataOp = (Va->item == rENTRY_NUMSTRINGS_);
+    struct CDFstruct *CDF;
+    long *value = va_arg (Va->ap, long *);
+    Int32 tValue;
+    OFF_T eOffset;
+    SelectCDF (Cur->cdf, CDF)
+    if (!CURRENTattrSELECTED64(CDF)) return NO_ATTR_SELECTED;
+    if (BOO(dataOp, CDF->CURgrEntryNum, CDF->CURzEntryNum) == 
+        RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
+    if (!sX(CheckEntryOp64(CDF,Va->item),&pStatus)) return pStatus;
+    eOffset = BOO(dataOp, CDF->CURgrEntryOffset64,
+			  CDF->CURzEntryOffset64);
+    if (eOffset == (OFF_T) RESERVED_ENTRYOFFSET64) return NO_SUCH_ENTRY;
+    if (!sX(ReadAEDR64(CDF->fp,eOffset,
+		       AEDR_NUMSTRINGS,&tValue,
+		       AEDR_NULL),&pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    *value = (long) BOO((tValue>0), tValue, 1);
+    break;
+  }
+  /****************************************************************************
+   * gENTRY_DATA_/rENTRY_DATA_/zENTRY_DATA_, 
   ****************************************************************************/
   case gENTRY_DATA_:
   case rENTRY_DATA_:
@@ -1891,6 +1930,47 @@ switch (Va->item) {
     break;
   }
   /****************************************************************************
+  * rENTRY_STRINGSDATA_/zENTRY_STRINGSDATA_, 
+  ****************************************************************************/
+  case rENTRY_STRINGSDATA_:
+  case zENTRY_STRINGSDATA_: {
+    Logical zOp = (Va->item == zENTRY_STRINGSDATA_);
+    struct CDFstruct *CDF;
+    Int32 dataType, numElems;
+    OFF_T offset;
+    long *numStrings = va_arg (Va->ap, long *);
+    char ***strings = va_arg (Va->ap, char ***);
+    char *tmp;
+    SelectCDF (Cur->cdf, CDF)
+    if (!CURRENTattrSELECTED64(CDF)) return NO_ATTR_SELECTED;
+    if ((!zOp?CDF->CURgrEntryNum:CDF->CURzEntryNum) == RESERVED_ENTRYNUM)
+      return NO_ENTRY_SELECTED;
+    offset = (!zOp?CDF->CURgrEntryOffset64:CDF->CURzEntryOffset64);
+    if (offset == (OFF_T) RESERVED_ENTRYOFFSET64) return NO_SUCH_ENTRY;
+    if (!sX(ReadAEDR64(CDF->fp,offset,
+		       AEDR_DATATYPE,&dataType,
+		       AEDR_NUMELEMS,&numElems,
+		       AEDR_NULL),&pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    if (!STRINGdataType(dataType)) return TRY_TO_READ_NONSTRING_DATA;
+    tmp = (char *) malloc((size_t)numElems+1);
+    if (!sX(ReadAEDR64(CDF->fp,offset,
+		       AEDR_NUMSTRINGS,numStrings,
+		       AEDR_VALUE,tmp,
+		       AEDR_NULL),&pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    *(tmp+numElems) = '\0';
+    /* Pre-3.7.0, the field was used and filled with 0. */ 
+    if (*numStrings < 1) *numStrings = 1;
+    StringBreakIntoStrings (tmp, numElems, *numStrings, strings);
+    free (tmp);
+    break;
+  }
+  /****************************************************************************
   * Unknown item, must be the next function.
   ****************************************************************************/
   default: {
@@ -1900,3 +1980,4 @@ switch (Va->item) {
 }
 return pStatus;
 }
+
