@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                            Memory Functions.
 *
@@ -24,6 +29,10 @@
 *   V2.2  20-May-12, M Liu      Modified to remove mallopt calls as the 
 *                               allocated spaces stay in virtual memory (
 *                               within IDL).
+*   V2.3  09-Aug-13, M Liu      Added cdfid_AllocateMemory, cdfid_getCDFid,
+*                               cdfid_FreeMemory for each open/created 64-bit
+*                               CDF id when called from Fortran as it uses
+*                               only 4-byte to keep the CDFid.
 ******************************************************************************/
 
 #define CDFMEM
@@ -35,7 +44,11 @@
 
 #define MEMLOG		0
 #define TO_STDOUT	0
-
+/*
+static long numAllocs = 0; 
+static long numAlloc2 = 0; 
+static long numAlloc3 = 0; 
+*/
 /******************************************************************************
 * MEM structures/typedef's/global variables.
 ******************************************************************************/
@@ -48,6 +61,13 @@ typedef struct memSTRUCT {	/* Structure. */
 } MEM;
 typedef MEM *MEMp;		/* Pointer (to structure). */
 static MEMp memHeadP = NULL;	/* Head of memory linked list. */
+
+typedef struct CDFidSTRUCT {	/* Structure. */
+  void *ptr;
+  struct CDFidSTRUCT *next;
+} CDFidMEM;
+typedef CDFidMEM *CDFidMEMp;	/* Pointer (to structure). */
+static CDFidMEMp cdfidHeadP = NULL;	/* Head of cdfid memory linked list. */
 #endif
 
 /******************************************************************************
@@ -60,7 +80,7 @@ static void TermVMemory PROTOARGs((void));
 #endif
 
 #if MEMLOG
-static uLong TotalBytes PROTOARGs((MEMp memHeadP));
+static uLongx TotalBytes PROTOARGs((MEMp memHeadP));
 #endif
 
 /******************************************************************************
@@ -72,15 +92,15 @@ static uLong TotalBytes PROTOARGs((MEMp memHeadP));
 #define ALLOCFAILED(type,which,nBytes) \
   {FILE *fp = FOPEN("mem.log","a"); \
   fprintf (fp, "%sllocation FAILED [%d]: %lu bytes\n", \
-	   type, which, (uLong) nBytes); \
+	   type, which, (uLongx) nBytes); \
   fprintf (stdout, "%sllocation FAILED [%d]: %lu bytes\n", \
-	   type, which, (uLong) nBytes); \
+	   type, which, (uLongx) nBytes); \
   fclose (fp);}
 #else
 #define ALLOCFAILED(type,which,nBytes) \
   {FILE *fp = FOPEN("mem.log","a"); \
   fprintf (fp, "%sllocation FAILED [%d]: %lu bytes\n", \
-	   type, which, (uLong) nBytes); \
+	   type, which, (uLongx) nBytes); \
   fclose (fp);}
 #endif
 #else
@@ -91,17 +111,17 @@ static uLong TotalBytes PROTOARGs((MEMp memHeadP));
 #if MEMLOG
 #if TO_STDOUT
 #define ALLOCSUCCESS(type,nBytes,ptr,totalBytes) \
-  {FILE *fp = FOPEN("mem.log","a"); uLong totalBytesSave = totalBytes; \
+  {FILE *fp = FOPEN("mem.log","a"); uLongx totalBytesSave = totalBytes; \
   fprintf (fp, "%sllocated: %lu bytes at %p (%lu bytes total)\n", \
-	   type, (uLong) nBytes, ptr, totalBytesSave); \
+	   type, (uLongx) nBytes, ptr, totalBytesSave); \
   fprintf (stdout, "%sllocated: %lu bytes at %p (%lu bytes total)\n", \
-	   type, (uLong) nBytes, ptr, totalBytesSave); \
+	   type, (uLongx) nBytes, ptr, totalBytesSave); \
   fclose (fp);}
 #else
 #define ALLOCSUCCESS(type,nBytes,ptr,totalBytes) \
   {FILE *fp = FOPEN("mem.log","a"); \
   fprintf (fp, "%sllocated: %lu bytes at %p (%lu bytes total)\n", \
-	   type, (uLong) nBytes, ptr, totalBytes); \
+	   type, (uLongx) nBytes, ptr, totalBytes); \
   fclose (fp);}
 #endif
 #else
@@ -113,13 +133,13 @@ static uLong TotalBytes PROTOARGs((MEMp memHeadP));
 #if TO_STDOUT
 #define FREESUCCESS(type,ptr,nBytes) \
   {FILE *fp = FOPEN("mem.log","a"); \
-  fprintf (fp, "Freed (%s): %p, %lu bytes\n", type, ptr, (uLong) nBytes); \
-  fprintf (stdout, "Freed (%s): %p, %lu bytes\n", type, ptr, (uLong) nBytes); \
+  fprintf (fp, "Freed (%s): %p, %lu bytes\n", type, ptr, (uLongx) nBytes); \
+  fprintf (stdout, "Freed (%s): %p, %lu bytes\n", type, ptr, (uLongx) nBytes); \
   fclose (fp);}
 #else
 #define FREESUCCESS(type,ptr,nBytes) \
   {FILE *fp = FOPEN("mem.log","a"); \
-  fprintf (fp, "Freed (%s): %p, %lu bytes\n", type, ptr, (uLong) nBytes); \
+  fprintf (fp, "Freed (%s): %p, %lu bytes\n", type, ptr, (uLongx) nBytes); \
   fclose (fp);}
 #endif
 #else
@@ -154,12 +174,14 @@ size_t nBytes;
 void (*fatalFnc) PROTOARGs((char *msg));
 {
 #if defined(BUILDINGforIDL)
+  /* ++numAllocs; */ /* printf("(+1)numAllocs=%ld\n",numAllocs); */
   return malloc(nBytes);
 #else
   MEMp mem;
+  /* ++numAllocs; */ /* printf("(+1)numAllocs=%ld\n",numAllocs); */
 #if defined(linux) || defined(__CYGWIN__)
-/*   mallopt(M_MMAP_MAX, 0); */
-/*   mallopt(M_TRIM_THRESHOLD, -1); */
+/*  mallopt(M_MMAP_MAX, 0); */
+/*  mallopt(M_TRIM_THRESHOLD, -1); */
 #endif
   if (nBytes < 1) {
     ALLOCFAILED("A",1,nBytes)
@@ -233,9 +255,11 @@ void *ptr;
 void (*fatalFnc) PROTOARGs((char *msg));
 {
 #if defined(BUILDINGforIDL)
+  /* --numAllocs; */ /* printf("(-1)numAllocs=%ld\n",numAllocs); */
   free (ptr);
   return 1;
 #else
+  /* --numAllocs; */ /* printf("(-1)numAllocs=%ld\n",numAllocs); */
   if (ptr == NULL) {
     int count = 0;
     MEMp mem = memHeadP;
@@ -268,12 +292,103 @@ void (*fatalFnc) PROTOARGs((char *msg));
       mem = mem->next;
     }
     FREEFAILED(ptr)
-    if (fatalFnc != NULL) (*fatalFnc)("Unable to free memory buffer.");
+    if (fatalFnc != NULL) (*fatalFnc)("Unable to free memory buffer 1.");
     return 0;
   }
 #endif
 }
 
+/******************************************************************************
+* cdfid_AllocateMemory.
+* Used for saving the CDFid pointer in 64-bit mode for Fortran interface.
+******************************************************************************/
+
+VISIBLE_PREFIX void cdfid_AllocateMemory (id, fatalFnc)
+void *id;
+void (*fatalFnc) PROTOARGs((char *msg));
+{
+  CDFidMEMp mem;
+  /* ++numAlloc3; */ /* printf("(+1)numAlloc3=%ld\n",numAlloc3); */
+  mem = (CDFidMEMp) malloc (sizeof(CDFidMEM));
+  if (mem == NULL) {
+    if (fatalFnc != NULL) (*fatalFnc)("Unable to allocate memory buffer [1].");
+    return;
+  }
+  mem->ptr = id;
+  mem->next = cdfidHeadP;
+  cdfidHeadP = mem;
+}
+
+/******************************************************************************
+* cdfid_getCDFid.
+* Return 8-byte CDF id based on the 4-byte information from Fortran interface.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFid cdfid_getCDFid (shortId, fatalFnc)
+Int32 shortId;
+void (*fatalFnc) PROTOARGs((char *msg));
+{
+  CDFidMEMp mem = cdfidHeadP;
+  union {
+    CDFid id;
+    Int32 i[2];
+  } u;
+  u.i[0] = u.i[1] = 0;
+  while (mem != NULL) {
+    u.id = mem->ptr;
+    if (u.i[0] == shortId || u.i[1] == shortId) return (CDFid) mem->ptr;
+    mem = mem->next;
+  }
+  return NULL;
+}
+
+/******************************************************************************
+* cdfid_FreeMemory.
+* If NULL is passed as the pointer to free, then free the entire list of
+* allocated memory blocks.
+******************************************************************************/
+
+VISIBLE_PREFIX int cdfid_FreeMemory (ptr, fatalFnc)
+void *ptr;
+void (*fatalFnc) PROTOARGs((char *msg));
+{
+  if (ptr == NULL) {
+    int count = 0;
+    CDFidMEMp mem = cdfidHeadP;
+    while (mem != NULL) {
+      CDFidMEMp memX = mem;
+      mem = mem->next;
+      FREESUCCESS("NULL",memX->ptr,8)
+      free (memX);
+      /* --numAlloc3; */ /* printf("(-1)numAlloc3=%ld\n",numAlloc3); */
+      count++;
+    }
+    cdfidHeadP = NULL;
+    return count;
+  }
+  else {
+    CDFidMEMp mem = cdfidHeadP, memPrev = NULL;
+    while (mem != NULL) {
+      if (mem->ptr == ptr) {
+	CDFidMEMp memX = mem;
+	if (memPrev == NULL)
+	  cdfidHeadP = mem->next;
+	else
+	  memPrev->next = mem->next;
+        FREESUCCESS("one",memX->ptr,8)
+	free (memX);
+        /* --numAlloc3; */ /* printf("(-1)numAlloc3=%ld\n",numAlloc3); */
+	return 1;
+      }
+      memPrev = mem;
+      mem = mem->next;
+    }
+    FREEFAILED(ptr)
+    if (fatalFnc != NULL) (*fatalFnc)("Unable to free memory buffer 2.");
+    return 0;
+  }
+}
+
 /******************************************************************************
 * cdf_FreeMemoryX.
 * If NULL is passed as the pointer to free, then free the entire list of
@@ -286,9 +401,11 @@ void (*fatalFnc) PROTOARGs((char *msg));
 int loc;
 {
 #if defined(BUILDINGforIDL)
+  /* --numAlloc2; */ /* printf("(-1)numAlloc2=%ld\n",numAlloc2); */
   free (ptr);
   return 1;
 #else
+  /* --numAlloc2; */ /* printf("(-1)numAlloc2=%ld\n",numAlloc2); */
   if (ptr == NULL) {
     int count = 0;
     MEMp mem = memHeadP;
@@ -323,7 +440,7 @@ int loc;
     FREEFAILED(ptr)
     if (fatalFnc != NULL) {
       char tmp[32+20+1];
-      strcpy (tmp, "Unable to free memory buffer at ");
+      strcpyX (tmp, "Unable to free memory buffer at ", 52);
       snprintf (EofS(tmp), (size_t) sizeof(tmp)-strlen(tmp), "%d", loc);
       (*fatalFnc)(tmp);
     }
@@ -346,9 +463,8 @@ void (*fatalFnc) PROTOARGs((char *msg));
 #if defined(BUILDINGforIDL)
   return calloc(nObjects,objSize);
 #else
-  size_t nBytes = nObjects * objSize, i;
+  size_t nBytes = nObjects * objSize;
   void *ptr = cdf_AllocateMemory (nBytes, fatalFnc);
-  /* if (ptr != NULL) for (i = 0; i < nBytes; i++) ((Byte *) ptr)[i] = 0; */
   if (ptr != NULL) memset (ptr, 0, nBytes);
   return ptr;
 #endif
@@ -363,7 +479,7 @@ void (*fatalFnc) PROTOARGs((char *msg));
 
 #if defined(MICROSOFTC_700) && INCLUDEvMEMORY
 static Logical InitVMemory () {
-  uInt maxParagraphs = (uInt) (vMemSize / 16);
+  uInt4 maxParagraphs = (uInt4) (vMemSize / 16);
   if (!_vheapinit(0U,maxParagraphs,vMemMask)) return FALSE;
   return TRUE;
 }
@@ -435,14 +551,27 @@ MemHandle handle;
 ******************************************************************************/
 
 #if MEMLOG
-static uLong TotalBytes (memHeadP)
+static uLongx TotalBytes (memHeadP)
 MEMp memHeadP;
 {
-  MEMp mem = memHeadP; uLong totalBytes = 0;
+  MEMp mem = memHeadP; uLongx totalBytes = 0;
   while (mem != NULL) {
-    totalBytes += (uLong) mem->nBytes;
+    totalBytes += (uLongx) mem->nBytes;
     mem = mem->next;
   }
   return totalBytes;
 }
 #endif
+
+
+/******************************************************************************
+* Display malloc/free counts.
+******************************************************************************/
+/*
+VISIBLE_PREFIX void ShowMemoryAllocCounts ()
+{
+  printf("*** cdf_AllocateMemory/cdf_FreeMemory: %d\n    cdfid_AllocateMemory/cdfid_FreeMemory: %d\n",
+         numAllocs,numAlloc3);
+}
+*/
+
