@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                     CDF library miscellaneous functions, part 1.
 *
@@ -213,7 +218,7 @@ struct CDFstruct *CDF;
 * it, the data will be properly preserved.
 ******************************************************************************/
 
-STATICforIDL CDFstatus UpdateDotCDF (CDF)
+VISIBLE_PREFIX CDFstatus UpdateDotCDF (CDF)
 struct CDFstruct *CDF;
 {
   CDFstatus pStatus = CDF_OK; int varN; struct VarStruct *Var; Logical zVar;
@@ -543,9 +548,9 @@ Int32 numElems;
 void *buffer;
 {
   CDFstatus pStatus = CDF_OK;
-  size_t nElemBytes = CDFelemSize(dataType);
+  size_t nElemBytes = (size_t) CDFelemSize(dataType);
   size_t nBytes = (size_t) (numElems * nElemBytes);
-  double eValue; Int32 elemN; Byte *ptr; void *tBuffer;
+  double eValue; Int32 elemN; Byte1 *ptr; void *tBuffer;
   /****************************************************************************
   * Try to encode/write entire buffer.
   ****************************************************************************/
@@ -713,9 +718,13 @@ Logical *upper;         /* Should extensions be upper case? */
 Logical *version;       /* Should a version number of `;1' be appended? */
 {
   char pathT[DU_MAX_PATH_LEN+1];
+  size_t pathLen;
+
+  pathLen = strlen (path);
 
   strcpyX (pathT, path, DU_MAX_PATH_LEN);
-  strcatX (pathT, ".cdf", DU_MAX_PATH_LEN);
+  if (pathLen < 4 || strcmp(pathT+(pathLen-4), ".cdf") != 0)
+    strcatX (pathT, ".cdf", DU_MAX_PATH_LEN);
   if (IsReg(pathT)) {
     *no_append = FALSE;
     *upper = FALSE;
@@ -724,7 +733,8 @@ Logical *version;       /* Should a version number of `;1' be appended? */
   }
 
   strcpyX (pathT, path, DU_MAX_PATH_LEN);
-  strcatX (pathT, ".CDF", DU_MAX_PATH_LEN);
+  if (pathLen < 4 || strcmp(pathT+(pathLen-4), ".CDF") != 0)
+    strcatX (pathT, ".CDF", DU_MAX_PATH_LEN);
   if (IsReg(pathT)) {
     *no_append = FALSE;
     *upper = TRUE;
@@ -733,7 +743,8 @@ Logical *version;       /* Should a version number of `;1' be appended? */
   }
 
   strcpyX (pathT, path, DU_MAX_PATH_LEN);
-  strcatX (pathT, ".cdf;1", DU_MAX_PATH_LEN);
+  if (pathLen < 6 || !StrStrIgCaseX(pathT+(pathLen-6), ".cdf;1"))
+    strcatX (pathT, ".cdf;1", DU_MAX_PATH_LEN);
   if (IsReg(pathT)) {
     *no_append = FALSE;
     *upper = FALSE;
@@ -742,7 +753,8 @@ Logical *version;       /* Should a version number of `;1' be appended? */
   }
 
   strcpyX (pathT, path, DU_MAX_PATH_LEN);
-  strcatX (pathT, ".CDF;1", DU_MAX_PATH_LEN);
+  if (pathLen < 6 || !StrStrIgCaseX(pathT+(pathLen-6), ".CDF;1"))
+    strcatX (pathT, ".CDF;1", DU_MAX_PATH_LEN);
   if (IsReg(pathT)) {
     *no_append = FALSE;
     *upper = TRUE;
@@ -750,17 +762,21 @@ Logical *version;       /* Should a version number of `;1' be appended? */
     return CDF_OK;
   }
 
-#if defined(unix) || defined(dos)
+/*
+#if defined(unix) || defined(dos) || defined(win32) 
+*/
+#if defined(win32) || defined(dos) 
   strcpyX (pathT, path, DU_MAX_PATH_LEN);
   MakeUpperString (pathT);
-  strcatX (pathT, ".CDF", DU_MAX_PATH_LEN);
+  if (pathLen < 4 || !StrStrIgCaseX(pathT+(pathLen-4), ".CDF"))
+    strcatX (pathT, ".CDF", DU_MAX_PATH_LEN);
   if (IsReg(pathT)) {
     *no_append = FALSE;
     *upper = TRUE;
     *version = FALSE;
     return CDF_OK;
   }
-
+/*
   strcpyX (pathT, path, DU_MAX_PATH_LEN);
   MakeUpperString (pathT);
   if (IsReg(pathT)) {
@@ -769,6 +785,7 @@ Logical *version;       /* Should a version number of `;1' be appended? */
     *version = FALSE;
     return CDF_OK;
   }
+*/
 #endif
 
   if (IsReg(path)) {
@@ -940,7 +957,7 @@ Int32 dimVarysP[];		/* Out: Dimension variances. */
 * length is reached.
 ******************************************************************************/
 
-STATICforIDL void NULterminateMAX (string, maxLen)
+VISIBLE_PREFIX void NULterminateMAX (string, maxLen)
 char *string;
 size_t maxLen;
 {
@@ -960,9 +977,7 @@ void *buffer;
 int firstByte;
 int lastByte;
 {
-  int i;
-  /* for (i = firstByte; i <= lastByte; i++) ((Byte *)buffer)[i] = 0; */
-  memset ((Byte *)buffer+firstByte, 0, (size_t) (lastByte - firstByte + 1));
+  memset ((Byte1 *)buffer+firstByte, 0, (size_t) (lastByte - firstByte + 1));
   return;
 }
 
@@ -1637,7 +1652,7 @@ struct CDFstruct *CDF;
        else {
 	 Int32 oldOffset = vOffset + VDR_WASTED_OFFSET + VDR_WASTED_SIZE,
 	       newOffset = vOffset + VDR_WASTED_OFFSET, byteX;
-	 Byte tByte;
+	 Byte1 tByte;
 	 for (byteX = 0; byteX < nTailBytes; byteX++) {
 	    if (!SEEKv(CDF->fp,oldOffset,vSEEK_SET)) return CDF_READ_ERROR;
 	    if (!READv(&tByte,1,1,CDF->fp)) return CDF_READ_ERROR;
@@ -1725,10 +1740,10 @@ struct CDFstruct *CDF;
       for (i = 0; i < 4; i++) {
 	 char aName[8+1];
 	 switch (i) {
-	   case 0: strcpy (aName, "VALIDMIN"); break;
-	   case 1: strcpy (aName, "VALIDMAX"); break;
-	   case 2: strcpy (aName, "SCALEMIN"); break;
-	   case 3: strcpy (aName, "SCALEMAX"); break;
+	   case 0: strcpyX (aName, "VALIDMIN", 8); break;
+	   case 1: strcpyX (aName, "VALIDMAX", 8); break;
+	   case 2: strcpyX (aName, "SCALEMIN", 8); break;
+	   case 3: strcpyX (aName, "SCALEMAX", 8); break;
 	 }
 	 tStatus = FindAttrByName (CDF,aName,&aOffset);
 	 switch (tStatus) {
@@ -1769,10 +1784,10 @@ struct CDFstruct *CDF;
 }
 
 /******************************************************************************
-* DeleteFile.
+* CDFdeleteFile.
 ******************************************************************************/
 
-STATICforIDL Logical DeleteFile (path)
+STATICforIDL Logical CDFdeleteFile (path)
 char *path;
 {
 #if defined(vms)
@@ -1831,8 +1846,10 @@ Logical deleteCDF;      /* Delete CDF file(s)? */
     /**************************************************************************
     * Close all of the CDF files if they are not already closed (or deleted).
     **************************************************************************/
-    if (CDF->dotFp != NULL) V_close (CDF->dotFp, CDF, NULL);
-    if (CDF->uDotFp != NULL) V_close (CDF->uDotFp, CDF, NULL);
+    if (CDF->dotFp != NULL) V_close (CDF->dotFp,
+				     ((updateCDF==UPDATE)?CDF:NULL), NULL);
+    if (CDF->uDotFp != NULL) V_close (CDF->uDotFp,
+				      ((updateCDF==UPDATE)?CDF:NULL), NULL);
     CloseVarFiles (CDF);
   }
   /****************************************************************************
@@ -1863,7 +1880,7 @@ struct CDFstruct *CDF;
   if (CDF->dotFp == NULL) {
     BuildFilePath (CDFt, CDF->CDFname, CDF->no_append, CDF->upper_case_ext,
 		   CDF->version_numbers, INT32_ZERO, tmpFile);
-    if (!DeleteFile(tmpFile)) sX (CDF_DELETE_ERROR, &pStatus);
+    if (!CDFdeleteFile(tmpFile)) sX (CDF_DELETE_ERROR, &pStatus);
   }
   else {
     if (!DELETEv(CDF->dotFp,NULL)) sX (CDF_DELETE_ERROR, &pStatus);
@@ -1886,7 +1903,7 @@ struct CDFstruct *CDF;
        }
        BuildFilePath (Vt, CDF->CDFname, CDF->no_append, CDF->upper_case_ext,
 		      CDF->version_numbers, varN, tmpFile);
-       if (!DeleteFile(tmpFile)) sX (VAR_DELETE_ERROR, &pStatus);
+       if (!CDFdeleteFile(tmpFile)) sX (VAR_DELETE_ERROR, &pStatus);
     }
     for (varN = 0; varN < CDF->NzVars; varN++) {
        struct VarStruct *Var = CDF->zVars[varN];
@@ -1899,7 +1916,7 @@ struct CDFstruct *CDF;
        }
        BuildFilePath (Zt, CDF->CDFname, CDF->no_append, CDF->upper_case_ext,
 		      CDF->version_numbers, varN, tmpFile);
-       if (!DeleteFile(tmpFile)) sX (VAR_DELETE_ERROR, &pStatus);
+       if (!CDFdeleteFile(tmpFile)) sX (VAR_DELETE_ERROR, &pStatus);
     }
   }
   return pStatus;
@@ -1995,23 +2012,23 @@ Int32 eOffset;
 ******************************************************************************/
 
 #if defined(MPW_C)
-STATICforIDL unsigned char *CtoPstr (string)
+STATICforIDL uChar *CtoPstr (string)
 char *string;
 {
   size_t length = MINIMUM(strlen(string),255);
   if (length > 0) memmove (&string[1], string, length);
   string[0] = (char) length;
-  return ((unsigned char *) string);
+  return ((uChar *) string);
 }
 #endif
 
 #if defined(MPW_C)
 STATICforIDL char *PtoCstr (string)
-unsigned char *string;
+uChar *string;
 {
   size_t length = (size_t) string[0];
   if (length > 0) memmove (string, &string[1], length);
-  string[length] = (unsigned char) NUL;
+  string[length] = (uChar) NUL;
   return ((char *) string);
 }
 #endif
@@ -2190,6 +2207,50 @@ long b;
   return MAXIMUM(a,b);
 }
 
+/******************************************************************************
+* FindNumberOfItems
+* Find the number of items in a string. Each item is delimited by the given
+* delimiter string.
+******************************************************************************/
+
+VISIBLE_PREFIX long FindNumberOfItems (string, numElems, delimiter)
+char *string;
+long numElems;
+char *delimiter;
+{
+  long count;
+  char *tmp, *tmp2, *tmp3;
+  int dlen = (int) strlen(delimiter);
+  tmp = malloc ((size_t) numElems+1);
+  memcpy (tmp, string, (size_t) numElems);
+  *(tmp+numElems) = '\0';
+  count = 1;
+  tmp2 = tmp;
+  tmp3 = strstr(tmp2, delimiter);
+  while (tmp3 != NULL) {
+    tmp2 = tmp3 + dlen;
+    ++count;
+    tmp3 = strstr(tmp2, delimiter);
+  }
+  free (tmp);
+  return count;
+}
+
+/******************************************************************************
+* CDF_Free_String
+* This is used to free the dynamic space allocated by the CDF library for
+* reading the string-based variable attribute entry.
+******************************************************************************/
+
+VISIBLE_PREFIX void CDF_Free_String (numStrings, strings)
+long numStrings;
+char **strings;
+{
+  int ix;
+  for (ix = 0; ix < (int) numStrings; ++ix) free(strings[ix]);
+  if (strings != NULL) free(strings);
+}
+
 /******************************************************************************
 * AddTOvStats.
 ******************************************************************************/
@@ -2223,6 +2284,56 @@ vSTATS *vStats;		/* If NULL, initialize (to zero). */
   return;
 }
 
+/******************************************************************************
+* FillSpacesToString.
+* This function scans for Nul in a string buffer. If found, it fills space(s)
+* until the end of the number of elements. This happens when a string is
+* entered with a length shorter than the defined number of elements. The
+* buffer's size is a number of numElms, which could contain multiple strings.
+******************************************************************************/
+STATICforIDL void FillSpacesToString (buffer, totalBytes, numElems)
+char  *buffer;
+int   totalBytes;
+int   numElems;
+{
+  int i, j, k, l;
+  l = totalBytes / numElems;
+  for (i = 0; i < l; ++i) {
+    for (j = 0; j < numElems; ++j) {
+      if (*(buffer+i*numElems+j) == '\0') {
+        for (k = j; k < numElems; ++k)
+          *(buffer+i*numElems+k) = ' ';
+        break;
+      }
+    }
+  }
+  return;
+}
+/******************************************************************************
+* FillNulsToString.
+* This function scans for Nul in a string buffer. If found, it fills Nul(s)
+* until the end of the number of elements. This happens when a string is
+* entered with a length shorter than the defined number of elements. The
+* buffer's size is a number of numElms, which could contain multiple strings.
+******************************************************************************/
+STATICforIDL void FillNulsToString (buffer, totalBytes, numElems)
+char  *buffer;
+int   totalBytes;
+int   numElems;
+{
+  int i, j, k, l;
+  l = totalBytes / numElems;
+  for (i = 0; i < l; ++i) {
+    for (j = 0; j < numElems; ++j) {
+      if (*(buffer+i*numElems+j) == '\0') {
+        for (k = j+1; k < numElems; ++k)
+          *(buffer+i*numElems+k) = '\0';
+        break;
+      }
+    }
+  }
+  return;
+}
 /******************************************************************************
 * DisplayVs.
 ******************************************************************************/
