@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                        CDF `open' operations.
 *
@@ -33,6 +38,8 @@
 #include "cdflib64.h"
 #include "cdfrev.h"
 
+static CDFstatus ValidateCDFTT2000 (CDFid id, int zVars, int rVars);
+
 /******************************************************************************
 * CDFope.
 ******************************************************************************/
@@ -56,6 +63,8 @@ switch (Va->item) {
     Logical upper_case_ext, version_numbers, no_append;
     Int32 magicNumber1, magicNumber2, CDRflags, version, release, increment;
     Logical largeFile;
+    int  validate;
+
     /**************************************************************************
     * Get arguments for this operation/item.
     **************************************************************************/
@@ -76,7 +85,7 @@ switch (Va->item) {
     MakeUpperString (CDFnameT);
 #endif
     RemoveCDFFileExtension(CDFnameT, CDFnameTx);
-    if (!ValidCDFname(CDFnameTx)) return BAD_CDF_NAME;
+/*  if (!ValidCDFname(CDFnameTx)) return BAD_CDF_NAME; */
     if (!sX(FindCDF(CDFnameTx,&no_append,
 		    &upper_case_ext,
 		    &version_numbers),&pStatus)) return pStatus;
@@ -86,18 +95,11 @@ switch (Va->item) {
     BuildFilePath (CDFt, CDFnameTx, no_append, upper_case_ext, version_numbers,
 		   INT32_ZERO, CDFnameX);
     if (!sX(CheckLFS(CDFnameX, &largeFile, NULL),&pStatus)) return pStatus; 
-
     if (!largeFile)
       dotFp = V_open (CDFnameX, READ_ONLY_a_mode);
     else
       dotFp = V_open64 (CDFnameX, READ_ONLY_a_mode);
     if (dotFp == NULL) return CDF_OPEN_ERROR;
-/*
-    if (dotFp->length > 0) 
-      largeFile = FALSE;
-    else
-      largeFile = TRUE;
-*/
     /**************************************************************************
     * Read the magic numbers.
     **************************************************************************/
@@ -138,7 +140,6 @@ switch (Va->item) {
       case V3magicNUMBER_1:
 	switch (magicNumber2) {
 	  case V2magicNUMBER_2u: /* Same as V3magicNUMBER_2u */
-/*	  case V3magicNUMBER_2u: */
 	    if (magicNumber1 == V2magicNUMBER_1) { /* 32-offset */
 	      if (!CACHEv(dotFp,NUMcacheUNKNOWN)) {
 	        V_close (dotFp, NULL, NULL);
@@ -153,7 +154,6 @@ switch (Va->item) {
 	    uDotFp = NULL;
 	    break;
 	  case V2magicNUMBER_2c:
-/*	  case V3magicNUMBER_2c: */
 	    uDotFp = V_scratch (ScratchDirectory(NULL), "cdf");
 	    if (uDotFp == NULL) {
 	      V_close (dotFp, NULL, NULL);
@@ -205,7 +205,8 @@ switch (Va->item) {
     /**************************************************************************
     * Allocate and begin initializing CDF structure.
     **************************************************************************/
-    CDF = (struct CDFstruct *) cdf_AllocateMemory (sizeof(struct CDFstruct), NULL);
+    CDF = (struct CDFstruct *) cdf_AllocateMemory (
+					(size_t)sizeof(struct CDFstruct), NULL);
     if (CDF == NULL) {
       if (!largeFile) {
         V_close (dotFp, NULL, NULL);
@@ -216,7 +217,8 @@ switch (Va->item) {
       } 
       return BAD_MALLOC;
     }
-    CDF->CDFname = (char *) cdf_AllocateMemory (strlen(CDFnameTx) + 1, NULL);
+    CDF->CDFname = (char *) cdf_AllocateMemory (
+					(size_t)strlen(CDFnameTx) + 1, NULL);
     if (CDF->CDFname == NULL) {
       if (!largeFile) {
         V_close (dotFp, NULL, NULL);
@@ -243,6 +245,7 @@ switch (Va->item) {
     CDF->zMode = zMODEoff;
     CDF->negToPosFp0 = FALSE;
     CDF->status = READ_ONLY;
+    CDF->tt2000Updated = 0;
     CDF->pseudo_clock = 0;
     CDF->stage.fp = NULL;
     CDF->stage.mark = ZERO_OFFSET;
@@ -289,8 +292,6 @@ switch (Va->item) {
 		      GDR_NrVARS,&(CDF->NrVars),
 		      GDR_NzVARS,&(CDF->NzVars),
 		      GDR_rMAXREC,&(CDF->rMaxRec),
-/*		      GDR_rNUMDIMS,&(CDF->rNumDims), */
-/*		      GDR_rDIMSIZES,CDF->rDimSizes, */
 		      GDR_NULL),&pStatus)) {
         AbortAccess (CDF, noUPDATE, noDELETE);
         cdf_FreeMemory (CDF, NULL);
@@ -345,7 +346,7 @@ switch (Va->item) {
     **************************************************************************/
     CDF->singleFile = SINGLEfileBITset (CDRflags);
     CDF->rowMajor = ROWmajorBITset (CDRflags);
-    CDF->checksum = (PriorTo("3.2.0",version,release,increment) ?
+    CDF->checksum = (PriorTo("3.0.0",version,release,increment) ?
                      NO_CHECKSUM : CDFChecksumMethod (CDRflags));
     CDF->fakeEPOCH = PriorTo("2.1.1",version,release,increment);
     CDF->wastedSpace = PriorTo("2.5",version,release,increment);
@@ -357,7 +358,8 @@ switch (Va->item) {
      * Validate the CDF to make sure that certain data fields are normal by
      * doing some sanity checks.
      ***********************************************************************/
-    if (CDFgetValidate()) {
+    validate = CDFgetValidate();
+    if (validate) {
       Logical debug;
       if (CDFgetValidateDebug())
         debug = TRUE;
@@ -389,7 +391,8 @@ switch (Va->item) {
                                    LIB_RELEASE_, &librel,
                                    LIB_INCREMENT_, &libinc,
                              NULL_);
-            if (PriorTo (vri, (Int32) libver, (Int32) librel, (Int32) libinc))
+            if (PriorTo(vri, (Int32)libver, (Int32)librel, (Int32)libinc) &&
+                PriorTo("3.5.0", (Int32)libver, (Int32)librel, (Int32)libinc))
               return UNABLE_TO_PROCESS_CDF;
             else
               return pStatus;
@@ -508,17 +511,32 @@ switch (Va->item) {
     **************************************************************************/
     Cur->cdf = CDF;
     *id = CDF;
-    if (CDF->singleFile && CDF->checksum != NO_CHECKSUM) {
-      if (!sX(CDFconfirmChecksum(*id),&pStatus)) return pStatus; 
-/*
-    } else {
-      int ev = CDFgetChecksumEnvVar();
-      if (CDF->checksum == NO_CHECKSUM && ev > 0) {
-        if (!sX(CDFsetChecksum(*id, (long) ev),&pStatus)) return pStatus;
+    if (validate && CDF->singleFile && CDF->checksum != NO_CHECKSUM) {
+      if (!sX(CDFconfirmChecksum(*id),&pStatus)) {
+        if (!largeFile) 
+          AbortAccess (CDF, noUPDATE, noDELETE);
+	else
+	  AbortAccess64 (CDF, noUPDATE, noDELETE);
+        cdf_FreeMemory (CDF, NULL);
+        Cur->cdf = NULL;
+        *id = (CDFid) NULL;
+        return pStatus;
       }
-*/
     }
-    AddOpenCDFsCount();
+    if (validate && (!PriorTo ("3.5.0", version, release, increment))) {
+        if (!sX(ValidateCDFTT2000 (*id, (int) CDF->NzVars, (int) CDF->NrVars),
+                &pStatus)) {
+          AbortAccess64 (CDF, noUPDATE, noDELETE);
+          cdf_FreeMemory (CDF, NULL);
+          Cur->cdf = NULL;
+          *id = (CDFid) NULL;
+          return pStatus;
+        }
+    }
+    if (!PriorTo ("3.5.0", version, release, increment)) {
+      CDFgetLeapSecondsTableStatus();
+      AddOpenCDFsCount();
+    }
     break;
   }
   /****************************************************************************
@@ -532,3 +550,108 @@ switch (Va->item) {
 
 return pStatus;
 }
+
+/******************************************************************************
+* ValidateCDFTT2000.
+******************************************************************************/
+
+static CDFstatus ValidateCDFTT2000 (CDFid id, int zVars, int rVars)
+{
+  long maxRec, dataType;
+  CDFstatus status;
+  int i, j, count, tt2000Sts;
+  long long tt2000 = DEFAULT_TT2000_PADVALUE;
+  void *buffer;
+  long year, month, day, tt2000LastUpdated, numDims, dimSizes[CDF_MAX_DIMS];
+  double yy, mm, dd, hh, mn, ss, ml, ms, ns;
+  int once;
+
+  status = CDFgetLeapSecondLastUpdated (id, &tt2000LastUpdated);
+  if (status != CDF_OK) return status;
+#if defined(vms)
+  CDFgetLastDateinLeapSecondsTBL (&year, &month, &day);
+#else
+  CDFgetLastDateinLeapSecondsTable (&year, &month, &day);
+#endif
+  if (tt2000LastUpdated == 0 ||
+      tt2000LastUpdated <= (10000*year+100*month+day)) return CDF_OK;
+  once = 0;
+  for (i = 0; i < zVars; ++i) {
+    status = CDFlib(SELECT_, zVAR_, (long) i,
+                    GET_, zVAR_DATATYPE_, &dataType,
+                    NULL_);
+    if (status != CDF_OK) return status;
+    if (dataType != CDF_TIME_TT2000) continue;
+    status = CDFlib(GET_, zVAR_MAXREC_, &maxRec,
+                    NULL_);
+    if (status != CDF_OK) return status;
+    if (maxRec < 0) continue;
+    /* If the file's last updated leap second date is later than table's, 
+       then, the table is out-dated. */
+    if ((once == 0) && (tt2000LastUpdated > (10000*year+100*month+day)))
+      return TT2000_USED_OUTDATED_TABLE;
+    else once = 1;
+    status = CDFlib(GET_, zVAR_NUMDIMS_, &numDims,
+                          zVAR_DIMSIZES_, dimSizes,
+                    NULL_);
+    if (status != CDF_OK) return status;
+    count = (int) (maxRec + 1);
+    if (numDims > 0)
+      for (j = 0; j < (int) numDims; ++j) count *= (int) dimSizes[j];
+    buffer = (void *) malloc ((size_t) (count * 8));
+    if (buffer == NULL) return BAD_MALLOC;
+    status =  CDFgetVarAllRecordsByVarID (id, 1, (long) i, buffer);
+    if (status < CDF_OK) {
+      free (buffer);
+      return status;
+    } 
+    for (j = 0; j < count; ++j) {
+      if (tt2000 < *(((long long *)buffer)+j))
+        tt2000 = *(((long long *)buffer)+j);
+    }
+    free (buffer);
+  }
+  status = CDFlib(GET_, rVARs_NUMDIMS_, &numDims,
+                        rVARs_DIMSIZES_, dimSizes,
+                  NULL_);
+  if (status != CDF_OK) return status;
+  for (i = 0; i < rVars; ++i) {
+    status = CDFlib(SELECT_, rVAR_, (long) i,
+                    GET_, rVAR_DATATYPE_, &dataType,
+                    NULL_);
+    if (status != CDF_OK) return status;
+    if (dataType != CDF_TIME_TT2000) continue;
+    status = CDFlib(GET_, rVAR_MAXREC_, &maxRec,
+                    NULL_);
+    if (status != CDF_OK) return status;
+    if (maxRec < 0) continue;
+    if ((once == 0) && (tt2000LastUpdated > (10000*year+100*month+day)))
+      return TT2000_USED_OUTDATED_TABLE;
+    else once = 1;
+    count = (int) (maxRec + 1);
+    if (numDims > 0) 
+      for (j = 0; j < (int) numDims; ++j) count *= (int) dimSizes[j];
+    buffer = (void *) malloc ((size_t) (count * 8));
+    if (buffer == NULL) return BAD_MALLOC;
+    status =  CDFgetVarAllRecordsByVarID (id, 0, (long) i, buffer);
+    if (status < CDF_OK) {
+      free (buffer);
+      return status;
+    }
+    for (j = 0; j < count; ++j) {
+      if (tt2000 < *(((long long *)buffer)+j))
+        tt2000 = *(((long long *)buffer)+j);
+    }
+    free (buffer);
+  }
+  if (tt2000 != DEFAULT_TT2000_PADVALUE) {
+    breakdownTT2000 (tt2000, &yy, &mm, &dd, &hh, &mn, &ss, &ml, &ms, &ns);
+    tt2000Sts = ValidateTT2000((int) (10000*yy+100*mm+dd),
+                               (int)tt2000LastUpdated);
+    if (tt2000Sts == -1) return TT2000_USED_OUTDATED_TABLE;
+    else if (tt2000Sts == 0) return TT2000_CDF_MAYNEEDUPDATE;
+  }
+  return CDF_OK;
+}
+
+
