@@ -236,7 +236,7 @@ struct STRINGstruct **ssh;		/* Pointer to head of STRINGstruct
 * returned.
 ******************************************************************************/
 
-STATICforIDL Logical Trailer (string, trailer)
+VISIBLE_PREFIX Logical Trailer (string, trailer)
 char *string;
 char *trailer;
 {
@@ -260,21 +260,17 @@ char *trailer;
 * including the terminating NUL character.
 ******************************************************************************/
 
-STATICforIDL char *strcpyX (dst, src, max)
+VISIBLE_PREFIX char *strcpyX (dst, src, max)
 char *dst;
 char *src;
 size_t max;
 {
   if (max > 0 && src[0] != NUL) {
-    size_t j;
-    j = MINIMUM(strlen(src), max);
+    size_t j, len;
+    len = strlen(src);
+    j = MINIMUM(max, len);
     strncpy (dst, src, j);
     dst[(int)j] = NUL;
-/*
-    int i;
-    for (i = 0; src[i] != NUL && i < (int) max; i++) dst[i] = src[i];
-    dst[i] = NUL;
-*/
   } else if (max == 0) {
     strcpy (dst, src);
   } else if (src[0] == NUL) {
@@ -298,7 +294,7 @@ size_t max;
 * including the terminating NUL character.
 ******************************************************************************/
 
-STATICforIDL char *strcatX (dst, src, max)
+VISIBLE_PREFIX char *strcatX (dst, src, max)
 char *dst;
 char *src;
 size_t max;
@@ -312,13 +308,6 @@ size_t max;
       if (k > 0) 
         strncat (dst, src, k);
     }
-/*
-    for (i = strlen(dst), j = 0; src[j] != NUL && i < (int) max; i++, j++) {
-       dst[i] = src[j];
-
-    }
-    dst[i] = NUL; 
-*/
   } else if (max == 0) {
     strcat (dst, src);
   }
@@ -334,7 +323,7 @@ size_t max;
 * concatenated. 
 ******************************************************************************/
 
-STATICforIDL char *catchrX (dst, src, max)
+VISIBLE_PREFIX char *catchrX (dst, src, max)
 char *dst;
 int src;
 size_t max;
@@ -358,7 +347,7 @@ size_t max;
 * destination string exceed a maximum length.
 ******************************************************************************/
 
-STATICforIDL char *prependX (dst, src, max)
+VISIBLE_PREFIX char *prependX (dst, src, max)
 char *dst;
 char *src;
 size_t max;
@@ -384,7 +373,7 @@ size_t max;
 {
   if (max < 0) return dst;
   int len = (int) BOO(max > 0,MINIMUM(max,(int)src[0]),src[0]);
-  strncpy (dst, &src[1], len);
+  strcpyX (dst, &src[1], len);
   dst[len] = NUL;
   return dst;
 }
@@ -451,14 +440,12 @@ int length;
     ptr = FORTstring;
     lenT = (size_t) length;
   }
-/*memmove (ptr, Cstring, lenT);*/
-  strncpy (ptr, Cstring, lenT);
+  strcpyX (ptr, Cstring, lenT);
   for (i = strlen(Cstring); i < lenT; i++) ptr[i] = ' ';
   return;
 #else
   int i;
-/*memmove (FORTstring, Cstring, (size_t) length);*/
-  strncpy (FORTstring, Cstring, (size_t) length);
+  strcpyX (FORTstring, Cstring, (size_t) length);
   for (i = strlen(Cstring); i < length; i++) ((char *) FORTstring)[i] = ' ';
   return;
 #endif
@@ -468,7 +455,7 @@ int length;
 * Printable.
 ******************************************************************************/
 
-STATICforIDL int Printable (c)
+VISIBLE_PREFIX int Printable (c)
 int c;
 {
   return BOO(IsASCII(c),isprint(c),FALSE);
@@ -478,7 +465,7 @@ int c;
 * UpperCase.
 ******************************************************************************/
 
-STATICforIDL int UpperCase (c)
+VISIBLE_PREFIX int UpperCase (c)
 int c;
 {
   return BOO(IsASCII(c),isupper(c),FALSE);
@@ -488,7 +475,7 @@ int c;
 * LowerCase.
 ******************************************************************************/
 
-STATICforIDL int LowerCase (c)
+VISIBLE_PREFIX int LowerCase (c)
 int c;
 {
   return BOO(IsASCII(c),islower(c),FALSE);
@@ -498,7 +485,7 @@ int c;
 * Alphabetic.
 ******************************************************************************/
 
-STATICforIDL int Alphabetic (c)
+VISIBLE_PREFIX int Alphabetic (c)
 int c;
 {
   return BOO(IsASCII(c),isalpha(c),FALSE);
@@ -508,7 +495,7 @@ int c;
 * Decimal.
 ******************************************************************************/
 
-STATICforIDL int Decimal (c)
+VISIBLE_PREFIX int Decimal (c)
 int c;
 {
   return BOO(IsASCII(c),isdigit(c),FALSE);
@@ -518,7 +505,7 @@ int c;
 * Spacing.
 ******************************************************************************/
 
-STATICforIDL int Spacing (c)
+VISIBLE_PREFIX int Spacing (c)
 int c;
 {
   return BOO(IsASCII(c),isspace(c),FALSE);
@@ -528,7 +515,7 @@ int c;
 * Visible.
 ******************************************************************************/
 
-STATICforIDL int Visible (c)
+VISIBLE_PREFIX int Visible (c)
 int c;
 {
   return BOO(IsASCII(c),isgraph(c),FALSE);
@@ -538,7 +525,7 @@ int c;
 * MakeLower.
 ******************************************************************************/
 
-STATICforIDL int MakeLower (c)
+VISIBLE_PREFIX int MakeLower (c)
 int c;
 {
   return BOO(UpperCase(c),(c | 0x20),c);
@@ -548,12 +535,60 @@ int c;
 * MakeUpper.
 ******************************************************************************/
 
-STATICforIDL int MakeUpper (c)
+VISIBLE_PREFIX int MakeUpper (c)
 int c;
 {
   return BOO(LowerCase(c),(c & 0x5F),c);
 }
 
+/******************************************************************************
+* CDFstrcmpIgCase.
+* It returns  1 if compared strings are identical with the same length
+*             0 if compared strings are not same or there is a NULL string
+******************************************************************************/
+
+int CDFstrcmpIgCase (string1, string2)
+char *string1;
+char *string2;
+{
+  size_t string1L, string2L;
+  int i;
+
+  if (string1 == NULL || string2 == NULL) return 0;
+  string1L = strlen(string1);
+  string2L = strlen(string2);
+  if (string1L != string2L) return 0;
+  for (i = 0; i < (int) string1L; i++) {
+     if (MakeLower(string1[i]) != MakeLower(string2[i]))
+       return 0;
+  }
+  return 1;
+}
+
+/******************************************************************************
+* CDFstrstrIgCase.
+******************************************************************************/
+
+char *CDFstrstrIgCase (string, substring)
+char *string;
+char *substring;
+{
+  size_t stringL, substringL;
+  int i, j;
+  
+  if (string == NULL || substring == NULL) return NULL;
+  stringL = strlen(string);
+  substringL = strlen(substring);
+  if (stringL == 0 || substringL == 0) return NULL;
+  if (stringL < substringL) return NULL;
+  for (i = 0; i < (int) stringL; i++) {
+     for (j = 0; j < (int) substringL; j++) {
+        if (MakeLower(substring[j]) != MakeLower(string[i+j])) break;
+     }
+     if (j == substringL) return &string[i];
+  }
+  return NULL;
+}
 
 /******************************************************************************
 * `strstr' (for SunOS before 4.1).  This should no longer be needed.  If it
