@@ -40,11 +40,11 @@ char *copyRight;
   strcpyX (copyRight,
 "\012\
 Common Data Format (CDF)\012\
-(C) Copyright 1990-2017 NASA/GSFC\012\
+https://cdf.gsfc.nasa.gov\012\
 Space Physics Data Facility\012\
 NASA/Goddard Space Flight Center\012\
 Greenbelt, Maryland 20771 USA\012\
-(Internet -- GSFC-CDF-SUPPORT@LISTS.NASA.GOV)\012", CDF_COPYRIGHT_LEN);
+(User support: gsfc-cdf-support@lists.nasa.gov)\012", CDF_COPYRIGHT_LEN);
   return;
 }
 
@@ -815,6 +815,18 @@ strcpyX (textPtr,
 CDF_STATUSTEXT_LEN);
 break;
 
+case TRY_TO_READ_NONSTRING_DATA:
+strcpyX (textPtr,
+"TRY_TO_READ_NONSTRING_DATA: Wrong function as the attribute entry data is non-string.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case BAD_NUM_STRINGS:
+strcpyX (textPtr,
+"BAD_NUM_STRINGS: Bad number of strings (at least 1 is required).",
+CDF_STATUSTEXT_LEN);
+break;
+
 default:
 snprintf (textPtr, (size_t) CDF_STATUSTEXT_LEN+1,
           "Text not found - unknown CDF status code (%ld).", status);
