@@ -196,6 +196,8 @@ va_dcl
 	 if (!Write64_64(fp,&(CDR->RecordSize))) return CWE;
 	 if (!Write32_64(fp,&(CDR->RecordType))) return CWE;
 	 if (!Write64_64(fp,&(CDR->GDRoffset))) return CWE;
+	 /* Group write for Version, Release, Encoding, Flags, rfuA, rfuB,
+	    Increment, rfuD, rfuE -- 9 4-byte big-endians. */
          if (!Write32s_64(fp,&(CDR->Version),9)) return CWE;
 /*
 	 if (!Write32_64(fp,&(CDR->Version))) return CWE;
@@ -284,6 +286,8 @@ va_dcl
 	 if (!SEEKv64(fp,offset,vSEEK_SET)) return CWE;
 	 if (!Write64_64(fp,&(GDR->RecordSize))) return CWE;
 	 if (!Write32_64(fp,&(GDR->RecordType))) return CWE;
+	 /* Group write for rVDRhead, zVDRhead, ADRhead, eof -- 4 8-byte
+	    big-endians. */
          if (!Write64s_64(fp,&(GDR->rVDRhead),4)) return CWE;
 /*
 	 if (!Write64_64(fp,&(GDR->rVDRhead))) return CWE;
@@ -291,6 +295,8 @@ va_dcl
 	 if (!Write64_64(fp,&(GDR->ADRhead))) return CWE;
 	 if (!Write64_64(fp,&(GDR->eof))) return CWE;
 */
+	 /* Group write for NrVars, NumAttr, rMaxRec, rNumDims, NzVars -- 5
+	    4-byte big-endians. */
          if (!Write32s_64(fp,&(GDR->NrVars),5)) return CWE;
 /*
 	 if (!Write32_64(fp,&(GDR->NrVars))) return CWE;
@@ -300,6 +306,8 @@ va_dcl
 	 if (!Write32_64(fp,&(GDR->NzVars))) return CWE;
 */
 	 if (!Write64_64(fp,&(GDR->UIRhead))) return CWE;
+	 /* Group write for rfuC, LeapSecondLastUpdated, rfuE -- 9 4-byte
+	    big-endians. */
          if (!Write32s_64(fp,&(GDR->rfuC),3)) return CWE;
 /*
 	 if (!Write32_64(fp,&(GDR->rfuC))) return CWE;
@@ -396,11 +404,14 @@ va_dcl
 	 if (!SEEKv64(fp,offset,vSEEK_SET)) return CWE;
 	 if (!Write64_64(fp,&(ADR->RecordSize))) return CWE;
 	 if (!Write32_64(fp,&(ADR->RecordType))) return CWE;
+	 /* Group write for ADRnext, AgrEDRhead -- 2 8-byte big-endians. */
          if (!Write64s_64(fp,&(ADR->ADRnext),2)) return CWE;
 /*
 	 if (!Write64_64(fp,&(ADR->ADRnext))) return CWE;
 	 if (!Write64_64(fp,&(ADR->AgrEDRhead))) return CWE;
 */
+	 /* Group write for Scope, Num, NgrEntries, MAXgrEntry, rfuA --
+	    5 4-byte big-endians. */
          if (!Write32s_64(fp,&(ADR->Scope),5)) return CWE;
 /*
 	 if (!Write32_64(fp,&(ADR->Scope))) return CWE;
@@ -410,6 +421,8 @@ va_dcl
 	 if (!Write32_64(fp,&(ADR->rfuA))) return CWE;
 */
 	 if (!Write64_64(fp,&(ADR->AzEDRhead))) return CWE;
+	 /* Group write for NzEntries, MAXzEntry, rfuE -- 3 4-byte
+	    big-endians. */
          if (!Write32s_64(fp,&(ADR->NzEntries),3)) return CWE;
 /*
 	 if (!Write32_64(fp,&(ADR->NzEntries))) return CWE;
@@ -503,6 +516,8 @@ va_dcl
 	 if (!Write64_64(fp,&(AEDR->RecordSize))) return CWE;
 	 if (!Write32_64(fp,&(AEDR->RecordType))) return CWE;
 	 if (!Write64_64(fp,&(AEDR->AEDRnext))) return CWE;
+	 /* Group write for AttrNum, DataType, Num, NumElems, NumStrings, rfuB,
+	    rfuC, rfuD, rfuE -- 9 4-byte big-endians. */
          if (!Write32s_64(fp,&(AEDR->AttrNum),9)) return CWE;
 /*
 
@@ -641,16 +656,20 @@ va_dcl
 	 if (!Write64_64(fp,&(VDR->RecordSize))) return CWE;
 	 if (!Write32_64(fp,&(VDR->RecordType))) return CWE;
 	 if (!Write64_64(fp,&(VDR->VDRnext))) return CWE;
+	 /* Group write for DataType, MaxRec -- 2 4-byte big-endians. */
          if (!Write32s_64(fp,&(VDR->DataType),2)) return CWE;
 /*
 	 if (!Write32_64(fp,&(VDR->DataType))) return CWE;
 	 if (!Write32_64(fp,&(VDR->MaxRec))) return CWE;
 */
+	 /* Group write for VXRhead, VXRtail -- 2 8-byte big-endians. */
          if (!Write64s_64(fp,&(VDR->VXRhead),2)) return CWE;
 /*
 	 if (!Write64_64(fp,&(VDR->VXRhead))) return CWE;
 	 if (!Write64_64(fp,&(VDR->VXRtail))) return CWE;
 */
+	 /* Group write for Flags, sRecords, rfuB, rfuC, rfuF, NumElems,
+	    Num -- 7 4-byte big-endians. */
          if (!Write32s_64(fp,&(VDR->Flags),7)) return CWE;
 /*
 	 if (!Write32_64(fp,&(VDR->Flags))) return CWE;
@@ -742,7 +761,7 @@ va_dcl
 			     VDR_zNUMDIMS,&nDims,
 			     VDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + (OFF_T) (zVDR_DIMVARYS_OFFSETb64 + 
-                                       (nDims * sizeof(Int32)));
+                                       (nDims * 4));
 	 }
 	 else {
 	   OFF_T GDRoffset;
@@ -771,8 +790,7 @@ va_dcl
 			     VDR_zNUMDIMS,&zNumDims,
 			     VDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + (OFF_T) (zVDR_PADVALUE_OFFSETb64 +
-		                       (zNumDims * sizeof(Int32)) + 
-                                       (zNumDims * sizeof(Int32)));
+		                       (zNumDims * 4) + (zNumDims * 4));
 	 }
 	 else {
 	   Int32 rNumDims; OFF_T GDRoffset;
@@ -783,7 +801,7 @@ va_dcl
 			     GDR_rNUMDIMS,&rNumDims,
 			     GDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + (OFF_T) (rVDR_PADVALUE_OFFSETb64 + 
-                                       (rNumDims*sizeof(Int32)));
+                                       (rNumDims*4));
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
 	 if (STRINGdataType(dataType)) {
@@ -882,6 +900,7 @@ va_dcl
 	 if (!Write64_64(fp,&(VXR->RecordSize))) return CWE;
 	 if (!Write32_64(fp,&(VXR->RecordType))) return CWE;
 	 if (!Write64_64(fp,&(VXR->VXRnext))) return CWE;
+	 /* Group write for Nentries, NusedEntries -- 2 4-byte big-endians. */
          if (!Write32s_64(fp,&(VXR->Nentries),2)) return CWE;
 /*
 	 if (!Write32_64(fp,&(VXR->Nentries))) return CWE;
@@ -901,7 +920,7 @@ va_dcl
 			   VXR_NULL),&pStatus)) return pStatus;
 	 switch (field) {
 	   case VXR_FIRSTREC: break;
-	   case VXR_LASTREC: tOffset += (OFF_T) nEntries * sizeof(Int32); break;
+	   case VXR_LASTREC: tOffset += (OFF_T) nEntries * 4; break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -914,7 +933,7 @@ va_dcl
          if (!sX(ReadVXR64(fp,offset,
                            VXR_NENTRIES,&nEntries,
                            VXR_NULL),&pStatus)) return pStatus;
-         tOffset += (OFF_T) (2 * nEntries * sizeof(Int32)); 
+         tOffset += (OFF_T) (2 * nEntries * 4); 
          if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
          if (!Write64s_64(fp,buffer,(int)nEntries)) return CWE;
          break;
@@ -1165,6 +1184,8 @@ va_dcl
 	 struct CPRstruct64 *CPR = va_arg (ap, struct CPRstruct64 *); int i;
 	 if (!SEEKv64(fp,offset,vSEEK_SET)) return CWE;
 	 if (!Write64_64(fp,&(CPR->RecordSize))) return CWE;
+	 /* Group write for RecordType, cType, rfuA, pCount -- 4 4-byte
+	    big-endians. */
          if (!Write32s_64(fp,&(CPR->RecordType),4)) return CWE;
 /*
 	 if (!Write32_64(fp,&(CPR->RecordType))) return CWE;
@@ -1237,6 +1258,8 @@ va_dcl
 	 struct SPRstruct64 *SPR = va_arg (ap, struct SPRstruct64 *); int i;
 	 if (!SEEKv64(fp,offset,vSEEK_SET)) return CWE;
 	 if (!Write64_64(fp,&(SPR->RecordSize))) return CWE;
+	 /* Group write for RecordType, sArraysType, rfuA, pCount -- 4 4-byte
+	    big-endians. */
          if (!Write32s_64(fp,&(SPR->RecordType),4)) return CWE;
 /*
 	 if (!Write32_64(fp,&(SPR->RecordType))) return CWE;
@@ -1309,6 +1332,7 @@ va_dcl
 	 struct CVVRstruct64 *CVVR = va_arg (ap, struct CVVRstruct64 *);
 	 if (!SEEKv64(fp,offset,vSEEK_SET)) return CWE;
 	 if (!Write64_64(fp,&(CVVR->RecordSize))) return CWE;
+	 /* Group write for RecordType, rfuA -- 2 4-byte big-endians. */
          if (!Write32s_64(fp,&(CVVR->RecordType),2)) return CWE;
 /*
 	 if (!Write32_64(fp,&(CVVR->RecordType))) return CWE;
