@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                            Virtual stream file.
 *
@@ -40,6 +45,9 @@
 *                               the actual read/write bytes is within the 
 *                               buffer size.
 *   V4.12 12-Aug-10, M Liu      Fill 0s to each new cache in AllocateBuffer.
+*   V4.13 10-Jan-17, M Liu      Modified to use mkstemp for creating temporary
+*                               file for Unix-based systems. Also, change the
+*                               directory to which the file will be created.
 *
 ******************************************************************************/
 
@@ -49,6 +57,9 @@
 
 #include "cdflib.h"
 #include "cdflib64.h"
+#if defined(win32)
+#  include "errno.h"
+#endif
 
 /******************************************************************************
 * Local macros/typedef's.
@@ -75,11 +86,13 @@ BOO(useVmem,LoadVMemory((MemHandle)cache->ptr,TRUE),cache->ptr)
 #define CACHEbufferWRITEto(cache) cache->ptr
 #endif
 
+#define TmpFileTemplate "mycdftmp.XXXXXX"
+
 /******************************************************************************
 * Local function prototypes.
 ******************************************************************************/
 
-static FILE *OpenFile PROTOARGs((char *file_spec, char *a_mode));
+static FILE *CDFOpenFile PROTOARGs((char *file_spec, char *a_mode));
 static Logical FreeCache PROTOARGs((vCACHE *firstCache));
 static vCACHE *FindCache PROTOARGs((vFILE *vFp, long blockN));
 static Logical vRead PROTOARGs((
@@ -101,10 +114,10 @@ static Logical ExtendFile PROTOARGs((vFILE *vFp, long toBlockN));
 #endif
 
 /******************************************************************************
-* OpenFile.
+* CDFOpenFile.
 ******************************************************************************/
 
-static FILE *OpenFile (file_spec, a_mode)
+static FILE *CDFOpenFile (file_spec, a_mode)
 char *file_spec;
 char *a_mode;
 {
@@ -220,14 +233,14 @@ vFILE *vFp;
   * reached.
   ****************************************************************************/
   if (vFp->nBuffers < vFp->maxBuffers) {
-    cache = (vCACHE *) cdf_AllocateMemory (sizeof(vCACHE), NULL);
+    cache = (vCACHE *) cdf_AllocateMemory ((size_t)sizeof(vCACHE), NULL);
     if (cache != NULL) {
 #if defined(MICROSOFTC_700) && INCLUDEvMEMORY
       if (useVmem)
 	cache->ptr = (void *) AllocateVMemory (nCACHE_BUFFER_BYTEs);
       else
 #endif
-	cache->ptr = cdf_AllocateMemory (nCACHE_BUFFER_BYTEs, NULL);
+	cache->ptr = cdf_AllocateMemory ((size_t)nCACHE_BUFFER_BYTEs, NULL);
       if (cache->ptr != NULL) {
 	if (vFp->cacheHead == NULL) {
 	  vFp->cacheHead = cache;
@@ -331,7 +344,7 @@ long toBlockN;
 	cache->modified = FALSE;
       }
       else {
-	Byte buffer[nCACHE_BUFFER_BYTEs];
+	Byte1 buffer[nCACHE_BUFFER_BYTEs];
 	if (!vRead(nCACHE_BUFFER_BYTEs * lastPhyBlockN,
 		   buffer,(size_t)nBytes,vFp)) return FALSE;
 #if CLEAR_BYTES
@@ -356,7 +369,7 @@ long toBlockN;
        cache->modified = FALSE;
      }
      else {
-       Byte buffer[nCACHE_BUFFER_BYTEs];
+       Byte1 buffer[nCACHE_BUFFER_BYTEs];
 #if CLEAR_BYTES
        ClearBytes (buffer, 0, nCACHE_BUFFER_BYTEs - 1);
 #endif
@@ -488,6 +501,30 @@ vFILE *vFp;
 #if defined(__MWERKS__)
   int ii;
 #endif
+  FILE *fp = NULL;
+  int writable;
+#if defined(unix) || defined(vms)
+  int fileid;
+#endif
+#if defined(win32)
+  int err;
+#endif
+  int pid;
+
+  pid = GetMyPID();
+  if (pid == -1) {
+#if defined(win32)
+    pid = (int) _getpid();
+#else
+# if !defined(unix)
+    pid = (int) getpid();
+# endif
+#endif
+    SetMyPID (pid);
+#if !defined(unix)
+    srand((unsigned int)pid*time(NULL));
+#endif
+  }
   /****************************************************************************
   * Create the scratch file if necessary.  If so, the current file path is
   * actually the scratch directory to be used.
@@ -495,28 +532,136 @@ vFILE *vFp;
   if (vFp->fp == NULL) {
     long i; char *tmpPath; size_t pathLength;
     pathLength = strlen(vFp->path) + 1 + 8 + 1 + EXT_LEN;
-    tmpPath = (char *) cdf_AllocateMemory (pathLength + 1, NULL);
+    if (pathLength < DU_MAX_DIR_LEN) pathLength = DU_MAX_DIR_LEN;
+    tmpPath = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
     if (tmpPath == NULL) return FALSE;
-    for (i = 1; i <= MAX_TMP; i++) {
-       strcpyX (tmpPath, vFp->path, 0);
-       AppendToDir (tmpPath, "");
-       snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
-                 "TMP%05ld.%s", i, vFp->scratchExt);
-       if (!IsReg(tmpPath)) {
-	 FILE *fp = OpenFile (tmpPath, WRITE_PLUS_a_mode);
-	 if (fp == NULL) {
-	   cdf_FreeMemory (tmpPath, NULL);
-	   return FALSE;
-	 }
-	 cdf_FreeMemory (vFp->path, NULL);
-	 vFp->path = tmpPath;
-	 vFp->fp = fp;
-	 break;
-       }
+    if (vFp->path != NULL && strlen(vFp->path) != 0) { 
+      strcpyX (tmpPath, vFp->path, 0);
+      AppendToDir (tmpPath, "");
+#     if defined(unix) || defined(vms)
+        strcatX (tmpPath, TmpFileTemplate, 0);
+        fileid = mkstemp(tmpPath);
+        if (fileid != -1) fp = fdopen(fileid, "w+");
+        else {
+          for (i = 1; i <= MAX_TMP; i++) {
+            strcpyX (tmpPath, vFp->path, 0);
+            AppendToDir (tmpPath, "");
+            snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+            if (!IsReg(tmpPath)) {
+              fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+              if (fp != NULL) break;
+            }
+          }
+        }
+#     else
+#       if defined(win32)
+          strcatX (tmpPath, TmpFileTemplate, 0);
+          err = _mktemp_s(tmpPath, (int)strlen(tmpPath)+1);
+          if (err == 0) {
+            fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+          } else {
+            if (err == EEXIST) {
+              int i1, i2;
+              for (i = 1; i <= MAX_TMP; i++) {
+                i1 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                i2 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                strcpyX (tmpPath, vFp->path, 0);
+                AppendToDir (tmpPath, "");
+                snprintf (EofS(tmpPath), (size_t) pathLength + 1 -
+                                         strlen(tmpPath),
+                          "TMP%c%c%5.5d.%s", i1, i2, rand(), vFp->scratchExt);
+                if (!IsReg(tmpPath)) {
+                  fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+                  if (fp != NULL) break;
+                }
+              }
+            }
+          }
+#       else
+          for (i = 1; i <= MAX_TMP; i++) {
+            strcpyX (tmpPath, vFp->path, (size_t)pathLength);
+            AppendToDir (tmpPath, "");
+            snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+            if (!IsReg(tmpPath)) fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+            if (fp != NULL) break;
+          }
+#       endif
+#     endif
+    } else {
+      char *tmpFolder = NULL;
+      strcpyX (tmpPath, "", (size_t)pathLength);
+#     if defined(unix) || defined(vms)
+        tmpFolder = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
+        if (tmpFolder == NULL) return FALSE;
+        writable = access(".", W_OK);
+        if (writable == 0) {
+          strcpyX (tmpPath, TmpFileTemplate, (size_t)pathLength);
+          strcpyX (tmpFolder, "", (size_t)pathLength);
+        } else {
+#         if defined(unix)
+            strcpyX (tmpPath, "/tmp/", (size_t)pathLength);
+            strcpyX (tmpFolder, tmpPath, (size_t)pathLength);
+#         else
+            strcpyX (tmpPath, "SYS$SCRATCH:", (size_t)pathLength);
+            strcpyX (tmpFolder, tmpPath, (size_t)pathLength);
+#         endif
+          strcatX (tmpPath, TmpFileTemplate, 0);
+        }
+        fileid = mkstemp(tmpPath);
+        if (fileid != -1) fp = fdopen(fileid, "w+");
+        else {
+          for (i = 1; i <= MAX_TMP; i++) {
+            strcpyX (tmpPath, tmpFolder, (size_t)pathLength);
+            snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+            if (!IsReg(tmpPath)) {
+              fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+              if (fp != NULL) break;
+            }
+          }
+        }
+        if (tmpFolder != NULL) cdf_FreeMemory (tmpFolder, NULL);
+#     else
+#       if defined(win32)
+          strcatX (tmpPath, TmpFileTemplate, 0);
+          err = _mktemp_s(tmpPath, (int)strlen(tmpPath)+1 );
+          if (err == 0) {
+            fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+          } else {
+            if (err == EEXIST) {
+              int i1, i2;
+              for (i = 1; i <= MAX_TMP; i++) {
+                strcpyX (tmpPath, "", (size_t)pathLength);
+                i1 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                i2 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                snprintf (EofS(tmpPath), (size_t) pathLength + 1 -
+                                         strlen(tmpPath),
+                          "TMP%c%c%5.5d.%s", i1, i2, rand(), vFp->scratchExt);
+                if (!IsReg(tmpPath)) {
+                  if (_access(tmpPath, 0) != 0) {
+                    fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+                    if (fp != NULL) break;
+                  }
+                }
+              }
+            }
+          }
+#       else
+          snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                    "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+          if (!IsReg(tmpPath)) fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+#       endif
+#     endif
     }
-    if (vFp->fp == NULL) {	   /* Hardly seems likely but we'll check... */
+    if (fp == NULL) {
       cdf_FreeMemory (tmpPath, NULL);
       return FALSE;
+    } else {
+      cdf_FreeMemory (vFp->path, NULL);
+      vFp->path = tmpPath;
+      vFp->fp = fp;
     }
   }
   /****************************************************************************
@@ -555,7 +700,7 @@ char *a_mode;           /* Access mode. */
   /****************************************************************************
   * Open the file.
   ****************************************************************************/
-  fp = OpenFile (file_spec, a_mode);
+  fp = CDFOpenFile (file_spec, a_mode);
   if (fp == NULL) return NULL;
 #if defined(vms)
   /****************************************************************************
@@ -603,7 +748,7 @@ char *a_mode;           /* Access mode. */
 	fclose (fp);
 	return NULL;
       }
-      fp = OpenFile (file_spec, a_mode);
+      fp = CDFOpenFile (file_spec, a_mode);
       if (fp == NULL) return NULL;
     }
   }
@@ -611,20 +756,19 @@ char *a_mode;           /* Access mode. */
   /****************************************************************************
   * Allocate and load vFILE structure.
   ****************************************************************************/
-  vFp = (vFILE *) cdf_AllocateMemory (sizeof(vFILE), NULL);
+  vFp = (vFILE *) cdf_AllocateMemory ((size_t)sizeof(vFILE), NULL);
   if (vFp == NULL) {
     fclose (fp);
     return NULL;
   }
   vFp->magic_number = VSTREAM_MAGIC_NUMBER;
   vFp->fp = fp;
-  vFp->path = (char *) cdf_AllocateMemory (strlen(file_spec) + 1, NULL);
+  vFp->path = (char *) cdf_AllocateMemory ((size_t)strlen(file_spec) + 1, NULL);
   if (vFp->path == NULL) {
     cdf_FreeMemory (vFp, NULL);
     fclose (fp);
     return NULL;
-  }
-  else
+  } else
     strcpyX (vFp->path, file_spec, 0);
   vFp->scratch = FALSE;
   vFp->error = FALSE;
@@ -699,18 +843,17 @@ char *extension;	/* Extension to use for the scratch file.  If NULL,
   /****************************************************************************
   * Allocate and load vFILE structure.
   ****************************************************************************/
-  vFp = (vFILE *) cdf_AllocateMemory (sizeof(vFILE), NULL);
+  vFp = (vFILE *) cdf_AllocateMemory ((size_t)sizeof(vFILE), NULL);
   if (vFp == NULL) return NULL;
   vFp->magic_number = VSTREAM_MAGIC_NUMBER;
   vFp->fp = NULL;
   vFp->fh = 0;
-  vFp->path = (char *) cdf_AllocateMemory (BOO(directory == NULL,
+  vFp->path = (char *) cdf_AllocateMemory ((size_t)BOO(directory == NULL,
 					   0,strlen(directory)) + 1, NULL);
   if (vFp->path == NULL) {
     cdf_FreeMemory (vFp, NULL);
     return NULL;
-  }
-  else
+  } else
     strcpyX (vFp->path, BOO(directory == NULL,"",directory), 0);
   strcpyX (vFp->scratchExt, BOO(extension == NULL,"ich",extension), EXT_LEN);
   vFp->scratch = TRUE;
@@ -736,7 +879,6 @@ char *extension;	/* Extension to use for the scratch file.  If NULL,
   vFp->GDR64 = NULL;
   vFp->ADRList = NULL;
   vFp->ADRList64 = NULL;
-
   /****************************************************************************
   * Return pointer to vFILE structure.
   ****************************************************************************/
@@ -888,7 +1030,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   long blockN;          /* Block number in file (from 0). */
   long atBlockN;        /* Block number in file (from 0) at which to read. */
   vCACHE *cache;        /* Pointer to cache structure. */
-  Byte *cBuffer;        /* Pointer to cache buffer. */
+  Byte1 *cBuffer;       /* Pointer to cache buffer. */
   int remainingBytes;	/* Number of bytes remaining in a block. */
   /****************************************************************************
   * Validate read.
@@ -936,7 +1078,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
       return 0;
     }
     memmove (buffer, cBuffer + bufferOffset, xBytes);
-    buffer = (Byte *) buffer + xBytes;
+    buffer = (Byte1 *) buffer + xBytes;
     atBlockN = firstBlockN + 1;
   }
   else
@@ -980,7 +1122,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 	 }
        }
      }
-     buffer = (Byte *) buffer + xBytes;
+     buffer = (Byte1 *) buffer + xBytes;
   }
   /****************************************************************************
   * Increment current file offset or set to EOF if the EOF indicator was set.
@@ -1008,7 +1150,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   long atBlockN;        /* Block number in file (from 0) at which to write. */
   size_t xBytes;        /* Number of bytes in a transfer. */
   vCACHE *cache;        /* Pointer to cache structure. */
-  Byte *cBuffer;        /* Pointer to cache buffer. */
+  Byte1 *cBuffer;       /* Pointer to cache buffer. */
   size_t nBytesInBlock;	/* Number of bytes to the end of the block. */
   /****************************************************************************
   * Validate write.
@@ -1072,7 +1214,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
     memmove (cBuffer + bufferOffset, buffer, xBytes);
     cache->modified = TRUE;
     vFp->length = MaxLong (vFp->length,(long) (vFp->offset + xBytes));
-    buffer = (Byte *) buffer + xBytes;
+    buffer = (Byte1 *) buffer + xBytes;
     atBlockN = firstBlockN + 1;
   }
   else
@@ -1149,7 +1291,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
      }
      vFp->length = MaxLong (vFp->length, 
 			    (long) ((nCACHE_BUFFER_BYTEs * blockN) + xBytes));
-     buffer = (Byte *) buffer + xBytes;
+     buffer = (Byte1 *) buffer + xBytes;
   }
   /****************************************************************************
   * Increment current file offset.
@@ -1266,9 +1408,8 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   * scratch file to which nothing has been written.
   ****************************************************************************/
   if (vFp->fp != NULL) {
-    if (CDF != NULL && (!CDF->readOnly || CDF->status == READ_WRITE) && 
+    if (CDF != NULL && !(CDF->readOnly || CDF->status == READ_ONLY) && 
         CDF->singleFile && (CDF->checksum != NONE_CHECKSUM)) {
-/*      if (!FLUSHv(vFp)) error = TRUE; */
       if (!CDFAddChecksum(CDF)) error = TRUE;
     }
     if (fclose(vFp->fp) == EOF) error = TRUE;
@@ -1323,7 +1464,7 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   * Delete the file (unless it was never created).
   ****************************************************************************/
   if (vFp->fp != NULL) {
-    if (!DeleteFile(vFp->path)) error = TRUE;
+    if (!CDFdeleteFile(vFp->path)) error = TRUE;
   }
   /****************************************************************************
   * Pass back statistics (if requested).
